#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
ctrl.py :
   Software controller manage interactions between elements 
"""
# standard library imports
import sys
import locale
import gettext
_ = gettext.gettext

# related third party imports
import gobject
import os.path

# local application/library specific imports
import luciole.base.app_logging as LOGGING
import luciole.base.exceptions as LEXCEP
import luciole.base.options as OPTIONS
import luciole.conf as LCONF
import luciole.constants as LCONST
import luciole.ctrl.constants as CTRL_CONST
import luciole.ctrl.import_image as CTRL_IMPORT
import luciole.ctrl.project_ctrl as CTRL_PROJECT
import luciole.ctrl.viewer_ctrl as CTRL_VIEWER
import luciole.gui.constants as GUI_CONST
import luciole.gui.gui_ctrl as GUI_CTRL
import luciole.info as INFO 
#####


class LucioleController(object):
    """ 
    Main luciole controller
    """

    def __init__(self, args) :
        """ Controller initialisation.
        Is the main appication intialisation """
        #
        # init attributes
        #
        self.ctrl_signals = None
        self.__rusher = None

        # parse program options 
        self.__options = OPTIONS.options_parser()
        
        # init logging
        self.logger = LOGGING.init_logging(
                            self.__options.is_verbose, 
                            log_to_file = self.__options.is_logfile
                            )
        self.logger.info("Starting luciole")
        
        # Init i18n 
        self.__init_i18n()
        
        # load configuration : ie. saved user options
        self.__configurer = LCONF.LucioleConf()
        #  load GUI theme
        self.__configurer.load_theme()
        # create controller signals
        self.__connect_signals()
        # initialize gui and BTW threads        
        self.__gui = GUI_CTRL.GuiController(self.ctrl_signals)
        


        # load recent projects
        _cbs = {'open-project' : self.on_open_project}
        _recent_mnger = self.__configurer.init_recent_manager(
                                   self.__gui.file_recent_menu,
                                   self.__gui.windows,
                                   _cbs)
        
        # prepare callbacks for Project controller
        _cbs = { 'done-rush' : self.on_done_rush,
                'close-project' : self.on_close_project }

        
        # start project controller
        self.__project = \
                CTRL_PROJECT.ProjectController(self.__gui,_recent_mnger ,_cbs)
        
        _cbs = {
                'on-done-snapshot' : self.on_done_snapshot,
                'on-error':self.on_error
                }
        self._viewer = CTRL_VIEWER.Viewer(self.__gui, _cbs)
        
        # display capture trash if option is set
        if  self.__configurer.conf_options['CaptureTrashDisplay'] == 'yes' :
            self.__gui.capture_trash = True



        # conncet gui signals after the controller initialisation
        self.__gui.connect_gui_signals()
        
    def __connect_signals(self):
        """ connect controller signals """
        self.ctrl_signals = gobject.GObject()
        for _signal in CTRL_CONST.CTRL_SIGNALS :
            
            # declare signal
            gobject.signal_new(_signal, self.ctrl_signals,
                       gobject.SIGNAL_RUN_LAST,
                       gobject.TYPE_NONE,
                       (gobject.TYPE_PYOBJECT,))

            # Build signal callback function name from signal name 
            # example : signal "change-alpha-mixer" 
            #           have callback self.on_change_alpha_mixer 
            _cb = 'self.on_' + _signal.replace('-', '_')
            # connect signal
            self.ctrl_signals.connect(_signal, eval(_cb))
            

    def on_change_alpha_mixer(self, *args) :
        """ Request change mixer alpha value """
        _args = self.__extract_signal_args(args) 
        try :
            self._viewer.change_mixer_alpha(_args['alpha'])
        except  LEXCEP.LucioException, _err_msg :
            raise LEXCEP.LucioException, _err_msg


    def on_change_framerate(self,  *args) :
        """ Request of framerate(fpi) change """
        _args = self.__extract_signal_args(args) 
        self.__project.change_framerate(_args['framerate']) 
        
    def on_change_project(self, *args) :
        """ Request for prohect change """
        _args = self.__extract_signal_args(args)
        for _key, _value in _args.iteritems() :
            self.__project.change_project(_key, _value)
        
    def on_close_project(self, *args ) :
        """ Request for project close  """
        _args = self.__extract_signal_args(args) 
        # test if project exists
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            # check if project is modified 
            if self.__project.data['is_modified'] :
                # ask for save it before close
                _res = self.__gui.windows.\
                    question_message(_('Save Project before closing'))
                if _res : 
                    self.__project.save()
                
        # set viewer in default mode 
        self._viewer.mode = self._viewer.DEFAULT
        
        # clear project controller 
        self.__project.mode = CTRL_CONST.NO_PROJECT
        
        # display close message 
        _msg = _('Project %s is closed' % self.__project.data['project_name'])
        self.__gui.status_bar.display_message(_msg)


    def on_create_project(self, *args) :
        """ Request new project """
        _args = self.__extract_signal_args(args) 
        self.__project.new()


    def on_delete_capture(self, *args) :
        """ Request delete on capture """
        _args = self.__extract_signal_args(args) 
        # remove on treeview
        self.__gui.treeviews[LCONST.CAPTURE].remove()
        
        # The image selected for delete is displayed before delete
        # so after go in default mode to nom more display the deleted image
        self._viewer.mode = self._viewer.DEFAULT
        
        # change control widget in no acquisition mode
        if self.__gui.control_widget.mode in\
                ( GUI_CONST.ACQUISITION_ACTIVE,
                GUI_CONST.ACQUISITION_ACTIVE_WITH_MIXER) :
            self.__gui.control_widget.mode = GUI_CONST.ACQUISTION_INACTIVE


    def on_delete_chrono(self, *args) :
        """ Request delete on capture """
        _args = self.__extract_signal_args(args) 
        # remove on treeview
        self.__gui.treeviews[LCONST.CHRONO].remove()
        
        # The image selected for delete is displayed before delete
        # so after go in default mode to nom more display the deleted image
        self._viewer.mode = self._viewer.DEFAULT

        # change control widget in no acquisition mode
        if self.__gui.control_widget.mode in \
                ( GUI_CONST.ACQUISITION_ACTIVE,
                GUI_CONST.ACQUISITION_ACTIVE_WITH_MIXER) :
            self.__gui.control_widget.mode = GUI_CONST.ACQUISTION_INACTIVE


    def on_display_about(self, *args) :
        """ request display about """
        _args = self.__extract_signal_args(args) 
        # compute version
        __version = "\n %s (%s)" % (INFO.VERSION, INFO.REVNO)
        # display window
        self.__gui.windows.about(__version)


    def on_display_luciole_preferences(self, *args) :
        """ Request display of application preferences """
        _args = self.__extract_signal_args(args) 
        _cbs = {'done-preferences' : self.on_done_preferences} 
        self.__gui.windows.preferences(  self.__configurer.conf_options,
                                        _cbs)
                                        
                                        
        


    def on_display_project_properties(self, *args) :
        """ Request display project properties """
        _args = self.__extract_signal_args(args) 
        # test if project exists
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            # go in default mode before dispalying window
            # due to webcam detection and avoid confilcts with acquisition
            self._viewer.mode = self._viewer.DEFAULT
            
            # change control widget in no acquisition mode
            self.__gui.control_widget.mode = GUI_CONST.ACQUISTION_INACTIVE

            # display message 
            self.__gui.status_bar.display_message(_('No Acquistion'))
            
            # launch project poperties window
            _cbs = \
            {'change-project-properties' : self.on_change_project_properties} 
            self.__gui.windows.project_properties(self.__project.data, _cbs) 


    def on_export_to_tool(self, *args) :
        """ Request export to tool  """
        _args = self.__extract_signal_args(args) 
        # test if project exists
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            self.__gui.windows.export_tool(self.__project.data)


    def on_export_to_video(self, *args) :
        """ Request export to video  """
        _args = self.__extract_signal_args(args) 
        # test if project exists
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            self.__gui.windows.export_video(self.__project.data)


    def on_import_image(self, *args) :
        """ Request image import """
        _args = self.__extract_signal_args(args) 
        # check if project exists
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            # open filename chooser dialog    
            _filenames = self.__gui.windows.import_dialog()
            
            if _filenames != [] :
                _cbs = { 'done-import' : self.on_done_import}
                # start import controller
                CTRL_IMPORT.ImportController(_filenames, 
                                            self.__project.data, 
                                            self.__rusher,
                                            self.__gui.status_bar,
                                            _cbs)
            else :
                _msg = _("No files or valid files choosen for image import.")
                self.__gui.windows.error_message( _msg)
        else :
            _msg = _("Impossible to import images when no project are loaded.")
            self.__gui.windows.error_message( _msg)
 
    def on_move_down_chrono(self, *args) :
        """ request to move down an image """
        _args = self.__extract_signal_args(args) 
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            self.__gui.treeviews[LCONST.CHRONO].move_down()


    def on_move_to_chrono(self, *args) :
        """  request for move imge from capture to chrono"""
        _args = self.__extract_signal_args(args) 
        # get images to move from capture widget
        _images_name = self.__gui.treeviews[LCONST.CAPTURE].images_to_move()
        # append this images in chrono widget
        _images_obj = \
            [ self.__rusher.get_image(_image) for _image in _images_name ]
        for _image in _images_obj :
            self.__gui.treeviews[LCONST.CHRONO].append_image(_image)
 

    def on_move_up_chrono(self, *args) :
        """ request to move up an image """
        _args = self.__extract_signal_args(args) 
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            self.__gui.treeviews[LCONST.CHRONO].move_up()


    def on_open_project(self, *args  ) :
        """ Request open project """
        _args = self.__extract_signal_args(args) 
        if 'path' in _args :
            if _args['path'] :
                # path is givan as param
                _path = _args['path']
            else :
                # launch open project window
                _path = self.__gui.windows.open_project()
        try :
            self.__project.open(_path)
        except LEXCEP.LucioException, _err_msg :
            _msg = _("Project load impossible\n%s" % _err_msg)
            self.__gui.windows.error_message( _msg)


    def on_play_video(self, *args) :
        """ Play video request """
        _args = self.__extract_signal_args(args) 
        # test if project exists
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            try :
                self._viewer.init_video_player(self.__project.data)
            except  LEXCEP.LucioException, _err_msg :
                raise LEXCEP.LucioException, _err_msg
            else :
                self._viewer.mode = self._viewer.PLAYER
                try :
                    self._viewer.play_video(self.__project.data)
                except  LEXCEP.LucioException, _err_msg :
                    raise LEXCEP.LucioException, _err_msg
        else :
            # send error message
            msg =  _("Can not play animation : No project loaded")
            self.__gui.windows.error_message( msg)


    def on_quit_app(self, *args) :
        """ Request quit application """
        _args = self.__extract_signal_args(args) 
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED and \
                self.__project.data['is_modified'] :
            _msg = _('Project modified. Save project before exit ?')
            _res = self.__gui.windows.question_message(_msg)
            # if response is not a bool : cancel clicked
            if isinstance(_res, bool) :
                if _res :
                    # if True save project before exit
                    self.__project.save()
                GUI_CTRL.GuiController.quit()
        else :
            GUI_CTRL.GuiController.quit()

    def on_save_as_project(self, *args) :
        """  save as project request """
        _args = self.__extract_signal_args(args) 
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            _dir = self.__gui.windows.dir_chooser()
            if _dir != None :
                # set viewer in default mode due to project reload
                self._viewer.mode = self._viewer.DEFAULT
                try : 
                    self.__project.save_as(_dir) 
                except LEXCEP.LucioException, _err_msg :
                    _msg = _('Impossible to save as project : %s' % _err_msg)
                    self.__gui.windows.error_message(_msg)
                else :
                    _msg = _('Project saved as %s' % \
                            self.__project.data['project_name'])
                    self.__gui.status_bar.display_message(_msg)


    def on_save_project(self, *args) :
        """ save project request """
        _args = self.__extract_signal_args(args) 
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            try : 
                self.__project.save() 
            except LEXCEP.LucioException, _err_msg :
                _msg = _('Impossible to save project : %s' % _err_msg)
                self.__gui.windows.error_message(_msg)
            else :
                _msg = _('Project %s saved' % \
                        self.__project.data['project_name'])
                self.__gui.status_bar.display_message(_msg)
                # update project name on Main bar
                self.__gui.set_program_bar(self.__project.data['project_name'],
                                        self.__project.data['is_modified'])


    def on_start_acquisition(self, *args ) :
        """Request for starting acqusisition """
        _args = self.__extract_signal_args(args) 
        # check if a project is loaded
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            self._viewer.mode = self._viewer.ACQUIRER
                       
            # change control widget in acquisition mode
            self.__gui.control_widget.mode = GUI_CONST.ACQUISITION_ACTIVE

        else :
            _msg = \
                _(' Can not start acquisition when no project are loaded.')
            self.__gui.windows.error_message(_msg)


    def on_start_mixer(self, *args) :
        """ Request mixer start """
        _args = self.__extract_signal_args(args) 
         # check if a project is loaded
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            # switch to viewer with acqusisition and mixer
            self._viewer.mode = self._viewer.ACQUIRER_WITH_MIXER
            # change control widget in no acquisition mode
            self.__gui.control_widget.mode = \
                    GUI_CONST.ACQUISITION_ACTIVE_WITH_MIXER

            # display message 
            self.__gui.status_bar.display_message(_('Acquiring with mixer'))
        else :
            # robustness
            _msg = \
                _('No project are loaded.')
            self.__gui.windows.error_message(_msg)



    def on_stop_acquisition(self, *args) :
        """ Request to stop acquistion  """
        _args = self.__extract_signal_args(args) 
        # check if a project is loaded
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            # switch to viewer default
            self._viewer.mode = self._viewer.DEFAULT 
            # change control widget in no acquisition mode
            self.__gui.control_widget.mode = GUI_CONST.ACQUISTION_INACTIVE

        else :
            # robustness
            _msg = \
                _('No project are loaded.')
            self.__gui.windows.error_message(_msg)

    def on_stop_mixer(self, *args) :
        """ Request mixer stop """
        _args = self.__extract_signal_args(args) 
         # check if a project is loaded
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            # switch to viewer with acqusisition and mixer
            self._viewer.mode = self._viewer.ACQUIRER
            # change control widget in no acquisition mode
            self.__gui.control_widget.mode = \
                    GUI_CONST.ACQUISITION_ACTIVE

            # display message 
            self.__gui.status_bar.display_message(_('Acquiring'))
        else :
            # robustness
            _msg = \
                _('No project are loaded.')
            self.__gui.windows.error_message(_msg)

    def on_stop_video(self, *args) :
        """ Stop video request  """
        _args = self.__extract_signal_args(args) 
        try :
            self._viewer.stop_video()
        except  LEXCEP.LucioException, _err_msg :
            raise LEXCEP.LucioException, _err_msg
        else :
            self._viewer.mode = self._viewer.DEFAULT


    def on_take_snapshot(self, *args) :
        """ take image snapshot request  """
        _args = self.__extract_signal_args(args) 
        if self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            self._viewer.take_snapshot()
        else :
            # robustness
            _msg = \
                _('No project are loaded.')
            self.__gui.windows.error_message(_msg)

            _msg = \
                _('No project are loaded.')
            self.__gui.windows.error_message(_msg)


    def on_view_image(self, *args) :
        """ 
        request image view/display :
        Mix image with stream if viewer mode is acquirer with mixer 
        """
        _args = self.__extract_signal_args(args) 
        #if mode is player does not allow image view
        if self._viewer.mode == self._viewer.PLAYER :
            pass
        
        # if mode is acquirer with mixer and image is from capture treeview :
        # mix image with acqusisition stream
        elif self._viewer.mode == self._viewer.ACQUIRER_WITH_MIXER \
        and _args['tv_type'] ==  LCONST.CAPTURE :
            self._viewer.mix_image_in_stream(_args['image'])
        
        # on other cases view image
        else :
            self._viewer.mode = self._viewer.IMAGER
            self._viewer.view_image(_args['image'])
            
            #change control widget in no acquisition mode
            if self.__gui.control_widget.mode in \
                    ( GUI_CONST.ACQUISITION_ACTIVE,
                    GUI_CONST.ACQUISITION_ACTIVE_WITH_MIXER) :
                self.__gui.control_widget.mode = GUI_CONST.ACQUISTION_INACTIVE


    def run(self) :
        """ main program ininite loop """
        GUI_CTRL.GuiController.threads_enter()
        # before starting main loop check for 
        # options given to application
        
        # load a project ? 
        if self.__options.filename \
        and os.path.exists(self.__options.filename):
            # emit open project signal
            self.ctrl_signals.emit(
                        "open-project",
                        {'path' : self.__options.filename}
                        )

        GUI_CTRL.GuiController.main_loop()
        GUI_CTRL.GuiController.threads_leave()
        self.logger.info("Leaving luciole")
    #
    #   Callbacks from controller
    #
    def on_done_rush(self, rusher) :
        """ callback : when tush images are loaded """
        if rusher != None \
        and self.__project.mode == CTRL_CONST.PROJECT_LOADED :
            # load treeviews
            # prepare data
            _app_data = {}
            _app_data['capture_images'] = self.__project.data['capture_images']
            _app_data['chrono_images'] = self.__project.data['chrono_images']
            _app_data['rush'] = rusher

            self.__gui.load_treeviews(_app_data) 
             
            # init acquisition 
            self._viewer.init_acquisition(self.__project.data)                 
            
            
            #  update fpi on gui and show it
            self.__gui.update_framerate(int(self.__project.data['fpi']))
            
            # show project  acquisition widgets
            self.__gui.control_widget.mode = GUI_CONST.ACQUISTION_INACTIVE 

            # update project name on Main bar
            self.__gui.set_program_bar(self.__project.data['project_name'],
                                        self.__project.data['is_modified'])
            
            # finish project load (project aspect)
            self.__project.finish_project_load()

            # store rusher
            self.__rusher = rusher
   
 

    def on_done_snapshot(self) :
        """
        callback : image snapshot is done.
        Now image can be processed into project
        """
        self.__project.append_snapshot(self.__rusher, self._viewer.acquirer)
        
    def on_done_preferences(self , modified_options) :
        """ callback : preferences modified fome diaog """
        # update prefernces
        (_capture_trash, _ask_restart) = \
            self.__configurer.update_preferences(modified_options)
        
        # check if gui update is needed
        if isinstance(_capture_trash, bool) :
            self.__gui.capture_trash = _capture_trash

        if isinstance(_ask_restart, bool) and _ask_restart == True :
            _msg = \
                _('Please restart Luciole to take into account the new theme ')
            self.__gui.status_bar.display_message(_msg)

    def on_change_project_properties(self, key, key_webcam= None, data =None) :
        """ callback project change from properties window """
        # webcam_data field of project dictionaty is a dictionaty
        # Test if webcam data change key
        if key == 'webcam_data':
            # make a local copy of webcam_data
            webcam_dict = self.__project.data['webcam_data']
            #test if webcam dict key exists
            if webcam_dict.has_key(key_webcam) :
                # update webcam key and call project change
                webcam_dict[key_webcam] = data
                self.__project.change_project('webcam_data', webcam_dict)

    def on_done_import(self, image_objs = [] ) :
        """ callback : image import (rush generation) is done """
        if image_objs != [] :
            self.__project.change_project(  'rush_images',
                                            self.__rusher.dump_image_name())
            # Not to be done in thread : interacts with gui
            for _image_obj in image_objs :
                self.__gui.append_capture(_image_obj)  
   

    
    #
    # Error calbacks 
    #
    def on_error(self, err_type, message) :
        """ Main error callback  """
        _msg = _("error type : %s.  %s" % (err_type, message))
        # Display error on log window
        self.logger.info(_msg)
    #
    # private methods
    #
    def __init_i18n(self) :
        """ intialize locales """
        _locale_path = LCONST.LOCALE_DIR

        # Init the list of languages to support
        _langs = []
        #Check the default locale
        _lc, _encoding = locale.getdefaultlocale()
        if (_lc):
            #If we have a default, it's the first in the list
            _langs = [_lc]
        # Now lets get all of the supported languages on the system
        _language = os.environ.get('LANGUAGE', None)
        _msg = "language : " , _language
        self.logger.debug(_msg)
        if (_language):
            # langage comes back something like en_CA:en_US:en_GB:en
            # on linuxy systems, on Win32 it's nothing, so we need to
            # split it up into a list
            _langs += _language.split(":")
        # Now add on to the back of the list the translations that we
        # know that we have, our defaults"""
        _langs += ["en_US"]
        
        # Now langs is a list of all of the languages that we are going
        # to try to use.  First we check the default, then what the system
        # told us, and finally the 'known' list
        _msg = "locale path", _locale_path
        self.logger.debug(_msg)
        
        GUI_CTRL.GuiController.init_18n(_locale_path)
        #import gtk.glade
        #gtk.glade.bindtextdomain(LCONST.APP_NAME, _locale_path)
        #gtk.glade.textdomain(LCONST.APP_NAME)

        gettext.bindtextdomain(LCONST.APP_NAME, _locale_path)
        gettext.textdomain(LCONST.APP_NAME)
        
        # Get the language to use
        _lang = gettext.translation(LCONST.APP_NAME, _locale_path
            , languages=_langs, fallback = True)
        # Install the language, map _() (which we marked our
        # strings to translate with) to self.lang.gettext() which will
        # translate them."""
        global _
        _ = _lang.gettext



    #
    # static_method
    #
    @staticmethod
    def __extract_signal_args(args) :
        """ 
        extract the signals arguments :
        arfs is a tuple
        The first argument is the emitter object ( gobject)
        the second argument is a dict with all the parameter given 
        to signal.
        
        The function returns the paramter dictionary
        """
        return args[1]
            

if __name__ == '__main__' :
    X = LucioleController(sys.argv)
    X.run() 
    


