#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
    Immport images from extenal sources.
    Interface class is ImportController
"""
# standard library imports
import os.path
import threading
import Queue
from gettext import gettext as _

# related third party imports
# N/A
import gobject

# local application/library specific imports
import luciole.base.exceptions as LEXCEP
import luciole.base.tools as LT
import luciole.constants as LCONST
import luciole.media.image as LI

class ImportThread(threading.Thread):
    """
    this thread is used to load the rush images, 
    its take a while so this action is threaded 
    """
    def __init__(self, filenames, project, rusher, queue) :
        """ initilise thread 
        """
        # init thread
        super(ImportThread, self).__init__()
        
        self.__filenames = filenames
        self.__pjt_data = project
        self.__rusher = rusher
        self.queue = queue
        
    def run(self) :
        """ run thread """
        # create rush list object
        _image_objs = []
        for (_index, _filename) in enumerate(self.__filenames) :
            # copy image to rush folder and resize it
            try : 
                _rush_image_name = \
                    self.__copy_import_to_rush_folder(_filename)
            except LEXCEP.LucioException, err :
                lerr =  "ERROR  on import image : " , err
                raise LEXCEP.LucioException, lerr
            else :
                # append image to rush list : 
                # usage of append_threaded special for thread execution  
                _image_objs.append(\
                    self.__rusher.append_threaded(_rush_image_name))
            
            
            # indicate progression on queue
            _qmsg = dict()
            _qmsg['progression'] = (_index + 1.0)/len(self.__filenames) 
            self.queue.put(_qmsg, block=False)

        
        # indicate finsih  on queue and send __rusher
        # send finsih message on queueu
        _qmsg = dict()
        _qmsg['finish'] = _image_objs
        self.queue.put(_qmsg, block=False)

    
    def __copy_import_to_rush_folder(self, p_image_path):
        """ copy imported image to rush dir : 
            1. copy image to tmp dir.
            2. resize it.
            3. move it to rush image dir """

        # build temp impage path
        _temp_dir = \
            os.path.join(self.__pjt_data['project_dir'], LCONST.TMP_DIR)
        # copy name
        _ac_image_temp = os.path.join(_temp_dir, LCONST.ACQUIRED_IMAGE_NAME)
        # resized copy name
        _ac_image_temp_rz = \
            os.path.join(_temp_dir,LCONST.ACQUIRED_IMAGE_NAME_RZ)

        # build rush image name
        _basename = LCONST.RUSH_FILENAME_TPL % self.__rusher.rush_index
        _rush_image = os.path.join(self.__pjt_data['project_dir'],
                                    self.__pjt_data['rush_dir'])
        _rush_image = os.path.join(_rush_image, _basename)

        try :  
            # 1. move image acquired image to tmp dir
            LT.copyf(p_image_path, _ac_image_temp)

            # 2. resize image result is in _ac_image_temp_rz
            _rz_obj = LI.ImageResize(_ac_image_temp, _ac_image_temp_rz )
            _rz_obj.convert()
            
            # 3. move resized image to rush dire 
            LT.movef(_ac_image_temp_rz, _rush_image)
        
        except LEXCEP.LucioException, _err :
            raise LEXCEP.LucioException, _err
        return _basename
 

class ImportController(object):
    """ This class is used to manage the import of image in project .
        The class use a Thread for importing images (it can takes) long.
        The thread and this class communicate via a queue.
        Information on queue are progression and finish.
        The queue is checked cyclicly via a gobject timer
    """
    _TIMEOUT = 50        # 50 ms timer

    def __init__(self, filenames, project_data, rusher, status_bar, cbs ) : 
        """ Init Thread, init gobject timer and clear progressbar """
        self.__filenames = filenames
        self.__pjt_data = project_data
        self.__rusher = rusher
        self.__status_bar = status_bar
        self.__cbs = cbs

        
        # initialize thread
        self.queue = Queue.Queue()
        self.t_rush = ImportThread(self.__filenames, 
                                    self.__pjt_data,
                                    self.__rusher,
                                    self.queue)
        
        #clear progressbar
        self._progress_bar_clear()
        
        # start Thread
        self.t_rush.start() 
        
        # start timer 
        gobject.timeout_add(self._TIMEOUT, self._on_timeout)
        
            

    def _on_timeout(self) :
        """ 
        function call peridodically by gobject timeout . The periodicity is ensured 
        by returning True. When False is return timer is stop  
        the goal of the time is too check the queue 
        """ 
        _rearm_timer = True
        try:
            _qmsg = self.queue.get(block = False)
        except Queue.Empty :
            pass
        else :
            if _qmsg.has_key('progression') : 
                self._progress_bar_on_progress(_qmsg['progression'])
            if _qmsg.has_key('finish') :
                # finisg project load ( Non threaded section )
                self.__cbs['done-import'](_qmsg['finish'])
                
                #update as finsih staus/progress bard
                self._progress_bar_complete()
                # stop the timer
                _rearm_timer = False
        
        return _rearm_timer
            
    def _progress_bar_clear(self):
        """ Clear progress bar  """
        _msg = _('Import started')
        self.__status_bar.start(_msg)

    def _progress_bar_complete(self):
        """ Progress bar full : Import done   """
        _msg = _('All images imported')
        # use of idle_add because gui update not in the same thread
        self.__status_bar.stop(_msg)

    def _progress_bar_on_progress(self, ratio = None):
        """ indicate that import  is going on """
        _msg = _('Importing images ...')
        self.__status_bar.on_progress(_msg, ratio)
 

   
    


