#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#

"""
    Load Rush images on application, i.e generates pixbuf
"""

# standard library imports
import os
import threading
import Queue
from gettext import gettext as _

# related third party imports
import gobject

# local application/library specific imports
import luciole.media.image as LI

class RushLoaderThread(threading.Thread):
    """ 
    this thread is used to load the rush images, 
    its take a while so this action is threaded 
    """
    def __init__(self, project, queue) :
        """ initilise thread 
            progress_bar : the status/progres bar for diplay info about project load 
            cb_on_finish : The callback function when thread is finished, the rush_obj is returned by thread 
            project : the project dictionary
        """
        # init thread
        super(RushLoaderThread, self).__init__()
        
        self.project = project
        self.queue = queue
        
    def run(self) :
        """ run thread """
        # create rush list object
        _rush_dir = os.path.join(self.project['project_dir'], 
                                self.project['rush_dir'])
        _rush_obj = LI.RushImages(  _rush_dir,
                                    self.project['rush_images'],
                                    self._cb_progression)
        
        # indicate finsih  on queue and send rush_obj
        # send finsih message on queueu
        _qmsg = dict()
        _qmsg['finish'] = _rush_obj
        self.queue.put(_qmsg, block=False)

    
    def _cb_progression(self, ratio = None):
        """ indicate progression of rsuh object init """
        # put in on queue
        _qmsg = dict()
        _qmsg['progression'] = ratio
        self.queue.put(_qmsg, block=False)


class ControllerLoadRush(object):
    """ This class is used to manage the load of a project in application.
        The class use a Thread for launch rush object init. (it can takes) long.
        The thread and this class communicate vai a queue. Information on queue are progression and finish.
        The queue is checked cyclicly via a gobject timer
    """
    _TIMEOUT = 10        # 10 ms timer

    def __init__(self, project, status_bar, cb_on_finish)  : 
        """ Init Thread, init gobject timer and clear progressbar """
        self.project = project
        self._cb_on_finish = cb_on_finish 

        self.__status_bar = status_bar
        
        # initialize thread
        self.queue = Queue.Queue()
        self.t_rush = RushLoaderThread(project, self.queue)
        
        #clear progressbar
        self._progress_bar_clear()
        
        # start Thread
        self.t_rush.start() 
        
        # start timer 
        gobject.timeout_add(self._TIMEOUT, self._on_timeout)
        
        #print " Controller load : ", threading.currentThread()
            

    def _on_timeout(self) :
        """ 
        function call peridodically by gobject timeout . The periodicity is ensured 
        by returning True. When False is return timer is stop  
        the goal of the time is too check the queue 
        """ 
        #print " on timeout : ", threading.currentThread()
        _rearm_timer = True
        try:
            _qmsg = self.queue.get(block = False)
        except Queue.Empty :
            pass
        else :
            if _qmsg.has_key('progression') : 
                self._progress_bar_on_progress(_qmsg['progression'])
            if _qmsg.has_key('finish') :
                # finisg project load ( Non threaded section )
                self._on_rush_finish(_qmsg['finish'])
                
                #update as finsih staus/progress bard
                self._progress_bar_complete()
                # stop the timer
                _rearm_timer = False
        
        return _rearm_timer
            
    def _progress_bar_clear(self):
        """ Clear progress bar  """
        _msg = _('Project Load started')
        self.__status_bar.start(_msg)

    def _progress_bar_complete(self):
        """ Progress bar full : Project loaded  """
        _msg = _( 'Project %s is loaded')%self.project['project_name'] 
        # use of idle_add because gui update not in the same thread
        self.__status_bar.stop(_msg)

    def _progress_bar_on_progress(self, ratio = None):
        """ indicate that detection  is going on """
        _msg = _('Project %s is loading')%self.project['project_name']
        self.__status_bar.on_progress(_msg, ratio)
 

    def _on_rush_finish(self, rush_obj):
        """ Finish the load after rush generation """
        # call back to indicate finish at controller
        self._cb_on_finish(rush_obj)
    
    


