#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
project_ctrl.py :
   Manage interactions with a luciole Project 
"""

# standard library imports
import os.path
from gettext import gettext as _

# related third party imports
# N/A

# local application/library specific imports
import luciole.base.exceptions as LEXCEP
import luciole.base.tools as LT
import luciole.constants as LCONST
import luciole.ctrl.constants as CTRL_CONST
import luciole.ctrl.ctrl_base as CTRL_BASE
import luciole.ctrl.load_rush as LRUSH
import luciole.gui.constants as GUI_CONST
import luciole.media.image as LI
import luciole.project.project as LPF

# NULL tuple constant defined for compat with gobject signals on ctrl.py file
_NULL_TUPLE = None, None

class ProjectController(CTRL_BASE.CtrlBase) :
    """ 
    Class to manage luciole project 
    SubClass of controller
    """
    #
    # CONSTANTS 
    #
    __MODES = (CTRL_CONST.NO_PROJECT, CTRL_CONST.PROJECT_LOADED)

    def __get_data(self):
        """ getter for is_project"""
        return self.__data
    data = property(  __get_data,
                            None,
                            None,
                            """Retrieve project data """
                            )   

    def __get_mode(self):
        """ getter for project controller mode """
        return self.__mode
    def __set_mode(self, mode) :
        """ setter for project controller mode """
        # change mode only on transition
        if mode in self.__MODES and mode != self.__mode :
            self.__mode_call[mode]()
    mode = property(	__get_mode, __set_mode, None,
						""" Project controller mode """)

    def __init__(self, gui, recent_mnger, cbs):
        """ 
            ctrl_cbs: dictionary of main main controller callbacks
        """
        CTRL_BASE.CtrlBase.__init__(self) 
        self.__data = None
        self.__mode = CTRL_CONST.NO_PROJECT
         # init mode 
        self.__mode_call = {
            CTRL_CONST.NO_PROJECT : \
                self.__set_mode_no_project,
             CTRL_CONST.PROJECT_LOADED : \
                self.__set_mode_project_loaded,
                }

        self.__gui = gui
        self.__gui_windows = gui.windows
        self.__cbs = cbs
        self.__recent_mngr = recent_mnger
        
        self.__pjt_file = LPF.ProjectToFile()

    def new(self) :
        """ create a new project """
        # if project exist : close it
        if self.__data != None :
            # (None, None) params for compat with gobject signal
            self.__cbs['close-project'](None, None)
        # launch New project assitant window
        self.__gui_windows.new_project_assistant(self._cb_new_project) 

    def open(self, project_path = None) :
        """ open an existing project """
        # no project path open a Dialog
        if project_path == None :
            _path =  self.__gui_windows.open_project()
        else :
            _path = project_path

        # if a project is loaded close it
        if self.__data != None :
            # (None, None) params for compat with gobject signal
            self.__cbs['close-project'](None, None)
        
        # check if a path entered ( from gui)
        if _path != None :
            try :
                (_is_valid, self.__data) =self.__pjt_file.open(_path)
            except LEXCEP.LucioException, _err_msg :
                # Transmit error to controller: not stop application
                raise LEXCEP.LucioException, _err_msg
            else :
                if not _is_valid :
                    _msg = _(" Webcam data not valid in project.\
                            Please restart webcam detection")
                    self.__gui_windows.error_message( _msg)
            
                # load project in application
                self.__load_project_in_app()


            
    def save_as(self, dir_path) :
        """ save as current project """
       
        # project saved and self.__data updated
        try :
            self.__pjt_file.save_as(dir_path , self.__data)
        except LEXCEP.LucioException, _err_msg :
            raise LEXCEP.LucioException, _err_msg
        else :
            self.__data['is_modified'] = False
            # saved project needs now to be reloaded
            # close it 
            self.__set_mode_no_project()
            # reload project

            self.__set_mode_project_loaded()
            self.__load_project_in_app()


    def save(self) :
        """ save current project """
        try :
            self.__pjt_file.save(self.__data)
        except LEXCEP.LucioException, _err_msg :
            raise LEXCEP.LucioException, _err_msg
        else :
            self.__data['is_modified'] = False





    def append_snapshot(self, rusher, acquirer) :
        """ move acquired  image to rush dir : 
            1. move image to tmp dir.
            2. resize it.
            3. move it to rush image dir """
        # get acquired image name
        l_acq_image = acquirer.image2save

        # build temp impage path
        l_temp_dir = os.path.join(self.__data['project_dir'], 'tmp')
        # copy name
        l_ac_image_temp = os.path.join(l_temp_dir, LCONST.ACQUIRED_IMAGE_NAME)
        # resized copy name
        l_ac_image_temp_rz = \
            os.path.join(l_temp_dir,LCONST.ACQUIRED_IMAGE_NAME_RZ)

        # build rush image name
        l_basename = LCONST.RUSH_FILENAME_TPL % rusher.rush_index
        l_rush_image = \
            os.path.join(self.__data['project_dir'], self.__data['rush_dir'])
        l_rush_image = os.path.join(l_rush_image, l_basename)

        try :  
            # 1. move image acquired image to tmp dir
            LT.movef(l_acq_image, l_ac_image_temp)

            # 2. resize image result is in l_ac_image_temp_rz
            l_rz_obj = LI.ImageResize(l_ac_image_temp, l_ac_image_temp_rz )
            l_rz_obj.convert()
            
            # 3. move resized image to rush dire 
            LT.movef(l_ac_image_temp_rz, l_rush_image)
        
        except LEXCEP.LucioException, _err_msg :
            raise LEXCEP.LucioException, _err_msg
        else : 
            # 4. append image to rush list
            rusher.append(l_basename)
            # indicate project change (rush list)
            self.__change_project('rush_images', rusher.dump_image_name())

             
            # 5. append image object to capture list
            l_rush_image = rusher.get_image(l_basename)
            # 6. always update the image 2 mix, even if mixer is not active
            # used to memorize the last capture 
            acquirer.Image2Mix = l_rush_image.path 
            
            #7 append capture to treeview
            self.__gui.append_capture(l_rush_image)  

    def change_framerate(self, fpi) :
        """ change framerate """
        # update framerate only if project exist
        if self.__data and isinstance(fpi, int): 
            self.__change_project('fpi', str(fpi))
   
    def change_project(self, project_key, data) :
        """ Change project : External mathod """
        self.__change_project(project_key, data)

    def finish_project_load(self) :
        """ 
            Method called to finsih project load.
        """
        # add project loaded to recent manager
        self.__recent_mngr.add_project(os.path.join(
                                self.__data['project_dir'],
                                self.__data['xml_filename'] )
                                )
        
        # Set project as not modified
        self.__data['is_modified'] =  False        
    #
    # def callbacks
    #
    def _cb_new_project(self, project_data) :
        """ callback when new project assistant is done """
        # 1. copy data in a project_dico object
        project_dico_data = LPF.project_dico() 
        for _key  in project_data.keys() :
            project_dico_data[_key] = project_data[_key]

        # 2. create project ( folder structure ,etc ...)
        # mising raise exception in case of create failure 
        try :
            self.__pjt_file.create(project_dico_data)
        except LEXCEP.LucioException, _err :
            raise LEXCEP.LucioException, _err
        else :
            self.__data = project_dico_data
        
            # 3. load created project in application
            self.__load_project_in_app()



    #
    # Private methods
    #
    def __set_mode_no_project(self): 
        """ No project mode """
        
        #
        # clear gui 
        #
        # clear treeviews 
        self.__gui.clear_treeviews()
        
        # set control widget in open_Load mode 
        self.__gui.control_widget.mode = GUI_CONST.LOAD_OR_CREATE

        # clear program bar
        self.__gui.set_program_bar('', False)
        
        
        self.__mode = CTRL_CONST.NO_PROJECT

    def __set_mode_project_loaded(self): 
        """project loaded mode """
        self.__mode = CTRL_CONST.PROJECT_LOADED

    def __save(self) :
        """ private method for save """
        pass

    def __load_project_in_app(self) :
        """ load project data in luciole :
            1. Load rusher
            2. when rusher done a callback is generated. 
                handled in main controller. Main controller
                load the acquistion object, and display
                project in gui
        """
        if self.__data != None :
            # set modified attibute 
            self.__data['is_modified'] = False
            
            # set set mode as PROJECT LOADED
            self.__mode = CTRL_CONST.PROJECT_LOADED
            
            self.logger.\
                debug("-----------------------------------------------------")
            self.logger.debug("Luciole_controller project info: ")
            self.logger.\
                debug("-----------------------------------------------------")
            for _key, _value in self.__data.iteritems() :
                self.logger.debug("**%s** : %s "%(_key, _value))
            self.logger.\
                debug("-----------------------------------------------------")

            # Initilaisation of rush obj is threaded because its take a while
            # (generation of images pixbufs)
            # When rush load is finish : __on_rush_finish is called 
            LRUSH.ControllerLoadRush(   self.__data,
                                        self.__gui.status_bar, 
                                        self.__cbs['done-rush'])


    def __change_project(self, project_key, data) :
        """ function called to update the current project """
        if self.__data != None :
            # is key exist ?
            if self.__data.has_key(project_key) :
                # update only on change
                if self.__data[project_key] != data :
                    self.__data[project_key] = data
                    self.__data['is_modified'] = True
                    self.__gui.set_program_bar( self.__data['project_name'], 
                                                self.__data['is_modified'])
            else :
                #raise Error/Exception 
                _err_msg =  "key %s not in project " % project_key 
                raise LEXCEP.LucioException, _err_msg
        else :
            # raise Error/Exception 
            _err_msg = " Project not loaded "
            raise LEXCEP.LucioException, _err_msg
 



