#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
viewer_ctrl.py :
   Viewer ctrl manage all views : 
	image, video, acquisitions snapshots 
"""
# standard library imports
import os.path
from gettext import gettext as _

# related third party imports

# local application/library specific imports
import luciole.base.exceptions as LEXCEP
import luciole.constants as LCONST
import luciole.ctrl.ctrl_base as CTRL_BASE
import luciole.media.acquisition as LACQ
import luciole.media.gst.play_sound as LSOUND
import luciole.media.image as LI
import luciole.media.player as LPLAYER

class Viewer(CTRL_BASE.CtrlBase) :
    """ 
    Manage the display on viewer widget.
    handle the selection of acquirer, imager, player,
    on the same window( drawarea widget)
    """ 
    #
    # Constants
    #
    (DEFAULT, IMAGER, PLAYER, ACQUIRER, ACQUIRER_WITH_MIXER) = range(5)
    __MODES = (DEFAULT, IMAGER, PLAYER, ACQUIRER, ACQUIRER_WITH_MIXER)
    
    __SOUND_FILE = '.data/sounds/camera.ogg'

    def __get_mode(self):
        """ getter for viewer mode """
        return self.__mode
    def __set_mode(self, mode) :
        """ setter for viewer mode """
        # change mode only on transition
        if mode in self.__MODES and mode != self.__mode :
            self.__mode_call[mode]()
    mode = property(	__get_mode, __set_mode, None,
						""" Viewer mode """)

    def __get_acquirer(self) :
        """ getter for acquirer """
        return self.__acquirer
    acquirer = property(__get_acquirer, None, None,
						""" Acquirer object """)


    def __init__(self,  gui, cbs ) :
        """ """
        # Base class init 
        CTRL_BASE.CtrlBase.__init__(self)
        
        self.__mode = self.IMAGER
        
        self.__gui = gui
        self.__viewer_widget = self.__gui.viewer_widget
        self.__cbs = cbs
        self.__acquirer = None
        self.__lucio_player = None

        # init mode 
        self.__mode_call = {
            self.DEFAULT : \
                self.__set_mode_default,
            self.IMAGER : \
                self.__set_mode_imager,
            self.PLAYER : \
                self.__set_mode_player,
            self.ACQUIRER : \
                self.__set_mode_acquirer,
            self.ACQUIRER_WITH_MIXER : \
                self.__set_mode_acquirer_with_mixer
                }

        #init sound player
        self.__sound_player = LSOUND.SoundPlayer(self.__SOUND_FILE)
        

    def init_acquisition(self, app_data = None) :
        """ initialize the acquirer   """
        if app_data != None :
            # Create acquisition object
            _acq_obj = None     
            # remark;  acquisition object for webcam and dv cam
            if app_data['hardtype'] == LCONST.WEBCAM :
                # acquisition for WEBCAM
                _acq_obj =  LACQ.AcquisitionWebcam(
                                    self.__viewer_widget.viewer_widget,
                                    False,
                                    data = app_data['webcam_data'],
                                    project_dir = app_data['project_dir'],
                                    cb_error = self._on_error_acqusition,
                                    cb_capture_done = self._on_done_snaphot)
            
            elif app_data['hardtype'] == LCONST.DVCAM :
                # default acquisition load i.e. DVCAM
                _acq_obj =  LACQ.Acquisition(
                                    self.__viewer_widget.viewer_widget,
                                    False,
                                    app_data['hardtype'],
                                    project_dir = app_data['project_dir'],
                                    cb_error = self._on_error_acqusition,
                                    cb_capture_done = self._on_done_snaphot)
            else :
                # Nothing to do
                pass
        
        if _acq_obj != None :
            # for mixer initialisation set image to mix with the last image 
            # of the capture view. only if capture image is not empty
            if len(app_data['capture_images'] ) > 0 :
                _l_last_image_path = \
                    os.path.join(   app_data['project_dir'],
                                    app_data['rush_dir'],
                                    app_data['capture_images'][-1])
            if os.path.exists(_l_last_image_path) : 
                _acq_obj.Image2Mix = _l_last_image_path

        self.__acquirer = _acq_obj

           

        
        

    def change_mixer_alpha(self, alpha) :
        """ change alpha value on mixer """
        if self.__mode == self.ACQUIRER_WITH_MIXER :
            if isinstance(alpha, float) :
                # check alpha range
                if alpha < 0.0 : 
                    alpha = 0.0
                if alpha > 1.0 : 
                    alpha = 1.0
                # update alpha on acquistion object
                self.__acquirer.set_alpha_onion_skin(alpha)
            else :
                _err_msg = "%s(%s) is not a float" % (alpha, type(alpha))
                raise LEXCEP.LucioException, _err_msg
        else :
            _err_msg = \
                "Invalid mode : mode should be ACQUIRER_WITH_MISER not %s" % \
                self.__mode
            raise LEXCEP.LucioException, _err_msg


    def take_snapshot(self) :
        """ request for take a snapshot from acquisition streaming """
        if self.__mode in (self.ACQUIRER, self.ACQUIRER_WITH_MIXER) :
            # check if acqusition remaisn active : 
            # i.e. No error from acquirer 
            if self.__acquirer.is_streaming_active == True :
                # set capture button insensitive
                self.__gui.is_button_snapshot_sensitive = False
                
                # play snapshot sound
                self.__sound_player.play()
                
                # request capture
                self.__acquirer.capture_image()
    
    def mix_image_in_stream(self, image) :
        """ mix an image with the stream """
        if self.__mode == self.ACQUIRER_WITH_MIXER :
            # check if image in correct format  
            if isinstance(image, LI.Image) :
                self.__acquirer.image2mix = image.path
            else :
                _err_msg = 'Image in incorrect format'
                raise LEXCEP.LucioException, _err_msg
        else :
            _err_msg = \
                "Invalid mode : mode should be ACQUIRER_WITH_MIXER not %s" % \
                self.__mode
            raise LEXCEP.LucioException, _err_msg
 

    def init_video_player(self, app_data) :
        """ intialize video player """
        
        # test if images are available in chrono zone
        if app_data['chrono_images'] != [] :
            # the play starts from this image to end
            try :
                # Initialize video player 
                self.__lucio_player =  \
                    LPLAYER.Player(
                        self.__viewer_widget.viewer_widget,
                        self._cb_player_finish,
                        os.path.join(   app_data['project_dir'], 
                                        LCONST.TMP_DIR )
                        )
            except LEXCEP.LucioException, _err :
                _err_msg = "Imposible to initialize player.\n", _err
                raise LEXCEP.LucioException, _err_msg
        else :
            # send error message
            _msg =  _("Can not play animation : No image on montage view ")
            self.__gui.window.error_message(_msg)


    def play_video(self, app_data) :
        """ play a video  from chrono images """
        if self.__mode == self.PLAYER :
                        
            # update play button with stop 
            self.__gui.play_video_button = "stop"
            # Init OK play can be started. 
            player_data = {}
            # creation of image list from  chrono_images the image names are 
            # listed, but the full path is need by palyer module
            # so each file is concated with absolute pathname to rush_dir
            player_data['image_list'] = \
                    [ os.path.join( app_data['project_dir'], 
                                    app_data['rush_dir'],
                                    image_name) \
                            for image_name in app_data['chrono_images'] ]
            
            # get position of selected image on chrono treeview, 
            _img_pos = \
                self.__gui.treeviews[LCONST.CHRONO].get_position_selected_row()

            #Slice with index_monatge to remove all images before selection
            player_data['image_list'] = \
                player_data['image_list'][_img_pos:]
            player_data['fpi'] = app_data['fpi']
            
            
            self.__lucio_player.start_play(player_data)

        else :
            _err_msg = \
                "Invalid mode : mode should be PLAYER not %s" % self.__mode
            raise LEXCEP.LucioException, _err_msg



    def stop_video(self) :
        """ stop video playing """
        
        # stop video is equvalent to switch in default mode
        self.__set_mode_default()


      
    def view_image(self, image ) :
        """ view an image from capture or chrono treewview """
        # mode should be imager 
        if self.__mode == self.IMAGER :
            # check if image in correct format  
            if  isinstance(image, LI.Image) :
                self.__gui.viewer_widget.image2display = image.pixbuf_normal
            else :
                _err_msg = 'Image in incorrect format'
                raise LEXCEP.LucioException, _err_msg
        else :
            _err_msg = \
                "Invalid mode : mode should be IMAGER not %s" % self.__mode
            raise LEXCEP.LucioException, _err_msg

    #
    # error calback
    #
    
    def _on_error_acqusition(self, message) :
        """
        callback error 
        propagate it to main controller.
        """
        print " acqusistion error\n ", message
        self.__cbs['on-error']("acqusisition", message)
    
    def _on_done_snaphot(self, *args) :
        """ 
        callback when snaphot done.
        propagate it to main controller.
        """ 
        # set capture button sensitive
        self.__gui.is_button_snapshot_sensitive = True

        self.__cbs['on-done-snapshot']()
    
    def _cb_player_finish(self) :
        """ callback executed when video play is finsihed """
        # Equivalent to stop video even if video is stopt
        self.__stop_video()
     

    #
    # private methods
    #
     
    def __set_mode_default(self) :
        """ set default mode """
        
        if self.__mode in (self.ACQUIRER, self.ACQUIRER_WITH_MIXER) :
            # stop acquistion
            self.__stop_acquisition()
        
        if self.__mode in (self.PLAYER,) :
            # stop video
            self.__stop_video()

        
        self.__viewer_widget.is_image_display = True
        
        # Display balck image
        self.__viewer_widget.image2display = None

        self.__mode = self.DEFAULT


    def __set_mode_imager(self) :
        """ set IMAGER mode """ 
        if self.__mode in (self.ACQUIRER, self.ACQUIRER_WITH_MIXER) :
            # stop acquistion
            self.__stop_acquisition()
        
        if self.__mode in (self.PLAYER,) :
            # stop video
            self.__stop_video()
        
        self.__viewer_widget.is_image_display = True
        # clear the image dispaly buffer :
        self.__viewer_widget.image2display = None
        
        self.__mode = self.IMAGER
   
    def __set_mode_player(self) :
        """ set PLAYER mode """
        if self.__mode in (self.DEFAULT, self.IMAGER ):
            # clear image  
            self.__viewer_widget.image2display = None
            # block image display
            self.__viewer_widget.is_image_display = False
        
        if self.__mode in (self.ACQUIRER, self.ACQUIRER_WITH_MIXER) :
            # stop acquistion
            self.__stop_acquisition()
        
        self.__mode = self.PLAYER

    def __set_mode_acquirer(self) :
        """ set ACQUIRER mode """
        if self.__mode in (self.DEFAULT, self.IMAGER ):
            # clear image  
            self.__viewer_widget.image2display = None
            # block image display
            self.__viewer_widget.is_image_display = False
        if self.__mode in (self.PLAYER, ) :
            # stop video
            self.__stop_video()
        if self.__mode in (self.ACQUIRER_WITH_MIXER, ) :
            self.__acquirer.deactive_onion_skin()
        
        self.__mode = self.ACQUIRER
        
        self.__start_acquisition()
        
   
    def __set_mode_acquirer_with_mixer(self) :
        """ set ACQUIRER_WITH_MIXER mode """
        if self.__mode in (self.DEFAULT, self.IMAGER ):
            # clear image  
            self.__viewer_widget.image2display = None
            # block image display
            self.__viewer_widget.is_image_display = False
        
        if self.__mode in (self.PLAYER,) :
            # stop video
            self.__stop_video()

        self.__mode = self.ACQUIRER_WITH_MIXER

        self.__start_acquisition()

    def __stop_acquisition(self) :
        """ stop the acquisition at acquirer level"""
        if self.__mode in (self.ACQUIRER, self.ACQUIRER_WITH_MIXER):
            if self.__mode == self.ACQUIRER_WITH_MIXER :
                self.__acquirer.deactive_onion_skin()
            self.__acquirer.stop_acquisition()
            # display message 
            self.__gui.status_bar.display_message(_('No Acquistion'))


    def __start_acquisition(self ) :            
        """ start the acquisition at acquirer level"""
        if self.__mode in (self.ACQUIRER, self.ACQUIRER_WITH_MIXER):
            if self.__mode == self.ACQUIRER_WITH_MIXER :
                self.__acquirer.active_onion_skin()
            self.__acquirer.start_acquisition()
            # display the acquiring message
            self.__gui.status_bar.display_message(_('Acquiring'))


    def __stop_video(self) :
        """ private function fior stop video """
        if self.__mode == self.PLAYER :
            # test if a player is on exectution 
            if self.__lucio_player != None :
                self.__lucio_player.stop_play()
        
            # show play button when video is stopt
            self.__gui.play_video_button = "play"
