#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
control_widget.py :
   Manage the control widget : This widget display
  the load or create buttons or the acqusisition buttons 
"""

# standard library imports
from gettext import gettext as _
import os.path
# related third party imports
import gtk

# local application/library specific imports
import luciole.base.exceptions as LEXCEP 
import luciole.constants as LCONST
import luciole.gui.constants as GCONST

class LoadOpenProjectWidget(gtk.HButtonBox) :
    """ widget for diplay New/open project buttons """

    # some constants for widget
    _WIDTH_REQUEST = -1
    _HEIGHT_REQUEST = 51        
    _IMAGE_POS = gtk.POS_TOP

    def __init__(self, ctrl_signals) :
        gtk.HButtonBox.__init__(self)
        
        self.__ctrl_signals = ctrl_signals

        # set widget properties
        self.set_name('hbox_open_project')
        self.set_size_request(self._WIDTH_REQUEST, self._HEIGHT_REQUEST)
        self.set_layout(gtk.BUTTONBOX_SPREAD)
        self.set_spacing(25)
        self.set_homogeneous(True)

        # create New button project
        msg = _('Create a new project')
        _button = gtk.Button(msg)
        _button.set_image(gtk.image_new_from_stock(  gtk.STOCK_NEW, 
                                                    gtk.ICON_SIZE_BUTTON))
        _button.set_property('image-position', self._IMAGE_POS)
        _button.connect('clicked', self.__on_new)
        self.pack_start(_button, expand = False, fill = False)
        _button.show()

        # create open button project
        msg = _('Open an existing project')
        _button = gtk.Button(msg)
        _button.set_image(gtk.image_new_from_stock(  gtk.STOCK_OPEN,
                                                    gtk.ICON_SIZE_BUTTON))
        _button.set_property('image-position', self._IMAGE_POS)
        _button.connect('clicked', self.__on_open)
        self.pack_start(_button, expand = False, fill = False)
        _button.show()
        
        # show widget 
        self.show()

    def __on_new(self, *args) :
        """ calback for create project """
        self.__ctrl_signals.emit('create-project', None)
    
    def __on_open(self, *args) :
        """ callback for opne project """
        self.__ctrl_signals.emit("open-project",  
                                {'path':None})

class ControlWidget(object) :
    """ Manage display of luciole control widget """
    #
    # CONSTANTS
    #
    __MODES = ( 
            GCONST.LOAD_OR_CREATE, 
            GCONST.ACQUISTION_INACTIVE, 
            GCONST.ACQUISITION_ACTIVE, 
            GCONST.ACQUISITION_ACTIVE_WITH_MIXER
            )
    
    __MAIN_CONTROL_WIDGET = 'hbox_acquisition'
    __ACQUISITION_BUTTON_IMAGE = 'acquisition_button_image'
    __SNAPSHOT_BUTTON = 'snapshot_button'
    __MIXER_BUTTON = 'mixer_checkbutton'
    __APLPHA_SCALE = 'alpha_hscale'
    __FRAMERATE_SCALE = 'hscale_fps'
    __MNU_PROJECT_PROP = 'view_project'
    __ACQUISITION_ACTION = 'acq_action'

    __PXB_SLEEP = os.path.join(LCONST.IMAGE_DIR,'luciole-dodo.png')
    __PXB_WAKEUP = os.path.join(LCONST.IMAGE_DIR,'luciole-eveillee.png')
    
    def __get_mode(self) :
        """ getter for mode """
        return self.__mode
    def __set_mode(self, mode) :
        """ setter function for mode """
        # change mode only on transition
        if mode  in self.__MODES and mode != self.__mode :
            self.__mode_call[mode]()
    mode = property(__get_mode,
                    __set_mode,
                    None,
                    """ Mode for control widget """
                    )
    
    def __init__(self, gui_widgets, ctrl_signals, priv_signals) :
        """ """
        self.__mode = GCONST.LOAD_OR_CREATE
        
        self.__gui_widgets = gui_widgets
        self.__ctrl_signals = ctrl_signals
        self._priv_signals = priv_signals
        
        # connect 'acquisition-toggled' signal
        self._priv_signals.connect("acquisition-toggled",
                                    self.on_acq_action_toggled)
        
        # Variable to identify origin of acquisition-toggled signal
        # Knows it is from gui or app. True indicates from app.
        self._from_app = False
        
        # acqusisition widget defined by default in glade
        self.__acqusition_widget = \
            self.__gui_widgets[self.__MAIN_CONTROL_WIDGET]
        self.__parent_widget = self.__acqusition_widget.get_parent()
        self.__load_or_create = \
            LoadOpenProjectWidget(ctrl_signals)

       
        # load pixbufs for acqusisition buttons
        self.__pixbufs = {}
        self.__pixbufs['sleep'] = \
            gtk.gdk.pixbuf_new_from_file(self.__PXB_SLEEP)
        self.__pixbufs['wake-up'] = \
            gtk.gdk.pixbuf_new_from_file(self.__PXB_WAKEUP)
        
        # intialise dict of fucntion according mode
        self.__mode_call = {
            GCONST.LOAD_OR_CREATE : \
                self.__load_or_create_mode,
            GCONST.ACQUISTION_INACTIVE : \
                self.__acqusition_inactive_mode,
            GCONST.ACQUISITION_ACTIVE : \
                self.__acqusition_active,
            GCONST.ACQUISITION_ACTIVE_WITH_MIXER : \
                self.__acquisition_active_with_mixer,
                }

        # by default mode is load or create project
        self.__load_or_create_mode()

    def __load_or_create_mode(self) :
        """ Load or Create project widget mode """
        # Project are not open in this mode : 
        # hide menu --> view --> project properties
        self.__gui_widgets[self.__MNU_PROJECT_PROP].set_sensitive(False)
        
        _current_widget = self.__gui_widgets[self.__MAIN_CONTROL_WIDGET]
        if _current_widget != None :
            # remove control widget
            self.__parent_widget.remove(_current_widget)
        # add load or create
        self.__parent_widget.add(self.__load_or_create)
        # update widget list
        self.__gui_widgets[self.__MAIN_CONTROL_WIDGET] = self.__load_or_create
        
        # update mode
        self.__mode = GCONST.LOAD_OR_CREATE
    
    def __acqusition_inactive_mode(self) :
        """ Acqusition inactive mode : in that mode a project is loaded """
        
        # action specific from load_or_create_mode to inactive
        if self.__mode == GCONST.LOAD_OR_CREATE :
            # show menu --> view --> project properties
            self.__gui_widgets[self.__MNU_PROJECT_PROP].set_sensitive(True)
        
            # change to acqusition widget
            _current_widget = self.__gui_widgets[self.__MAIN_CONTROL_WIDGET]
            if _current_widget != None :
                # remove control widget
                self.__parent_widget.remove(_current_widget)
            # add load or create
            self.__parent_widget.add(self.__acqusition_widget)
            # update widget list
            self.__gui_widgets[self.__MAIN_CONTROL_WIDGET] = \
                self.__acqusition_widget
        
        # Show acquisition button with a sleeping luciole
        self.__gui_widgets[self.__ACQUISITION_BUTTON_IMAGE]. \
            set_from_pixbuf(self.__pixbufs['sleep'])

        # Hide buttons 
        self.__gui_widgets[self.__SNAPSHOT_BUTTON].hide()
        self.__gui_widgets[self.__MIXER_BUTTON].hide()
        self.__gui_widgets[self.__APLPHA_SCALE].hide()
        
        # if acqusition button is active : inactivate it
        if self.__gui_widgets[self.__ACQUISITION_ACTION].get_active() == True :
            # the set_active function cause the toggle button signal emitted
            # origin of signal controled by _from_app
            self._from_app = True
            self.__gui_widgets[self.__ACQUISITION_ACTION].set_active(False)

        self.__mode = GCONST.ACQUISTION_INACTIVE
    
    def __acqusition_active(self) :
        """ Acuqusition active mode without mixer"""
        # previous mode can not be LOAD_OR_CREATE
        if self.__mode != GCONST.LOAD_OR_CREATE :
            # Show acquisition button with a wake-up luciole
            self.__gui_widgets[self.__ACQUISITION_BUTTON_IMAGE]. \
            set_from_pixbuf(self.__pixbufs['wake-up'])
            # show snapshot and mixer button
            self.__gui_widgets[self.__SNAPSHOT_BUTTON].show()
            # show checkbox for mixer inatctivated
            self.__gui_widgets[self.__MIXER_BUTTON].show()
            self.__gui_widgets[self.__MIXER_BUTTON].set_active(False)
            
            # Hide alpha scale button 
            self.__gui_widgets[self.__APLPHA_SCALE].hide()
            # if prevous mode is AVTIVE_WITH_MIXER : inactive check box :
            # if acqusition button is active : inactivate it
            if self.__gui_widgets[self.__ACQUISITION_ACTION].get_active() \
                    == False :
                # the set_active function cause 
                # the toggle button signal emitted
                # origin of signal controled by _from_app
                self._from_app = True
                self.__gui_widgets[self.__ACQUISITION_ACTION].set_active(True)
            self.__mode = GCONST.ACQUISITION_ACTIVE
        else :
            _err_msg = "Invalid transition mode.\
                    LOAD_OR_CREATE mode not valid"
            raise LEXCEP.LucioException, _err_msg

    def __acquisition_active_with_mixer(self) :
        """ Acuqusition active mode without mixer"""
        # previous mode can not be LOAD_OR_CREATE
        if self.__mode != GCONST.LOAD_OR_CREATE :
            # Show acquisition button with a wake-up luciole
            self.__gui_widgets[self.__ACQUISITION_BUTTON_IMAGE]. \
            set_from_pixbuf(self.__pixbufs['wake-up'])
            # show snapshot and mixer alpha button
            self.__gui_widgets[self.__SNAPSHOT_BUTTON].show()
            self.__gui_widgets[self.__MIXER_BUTTON].show()
            self.__gui_widgets[self.__APLPHA_SCALE].show()
            # if acqusition button is active : inactivate it
            if self.__gui_widgets[self.__ACQUISITION_ACTION].get_active() \
                    == False :
                # the set_active function cause 
                # the toggle button signal emitted
                # origin of signal controled by _from_app
                self._from_app = True
                self.__gui_widgets[self.__ACQUISITION_ACTION].set_active(True)


            self.__mode = GCONST.ACQUISITION_ACTIVE_WITH_MIXER
        else :
            _err_msg = "Invalid transition mode.\
                    LOAD_OR_CREATE mode not valid"
            raise LEXCEP.LucioException, _err_msg
 
    def on_acq_action_toggled(self, *args) :
        """ Acquisition button toggled """
        _widget = args[1]['widget']
        if not self._from_app :
            # Emit signal to control only when is a user request
            if _widget.get_active() :
                # Acqusistion button activated : start acquisition
                self.__ctrl_signals.emit("start-acquisition", None)
            else :
                # Acqusistion button de-activated : stop acquisition
                self.__ctrl_signals.emit("stop-acquisition", None)
        # clear flag
        self._from_app = False


