#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
export_video_window.py :
    Window for export to video
"""

# standard library imports
import logging 
import os.path
import re
from gettext import gettext as _

# related third party imports
import gtk

# local application/library specific imports
import luciole.gui.windows.dialog as G_DIALOG
import luciole.project.export.export_video as LEXP


class ExportVideoWindow(object) :
    """ This class manages the window used for video export"""
    
    ###########################################################################
    ### CONSTANTS
    ########################################################################### 
    
    __EXPORT_DIALOG     =   "export1" 
    __COMBO_BOX         =   "combobox4"
    __ENTRY_FILENAME    =   "entry1_filename"
    __FILECHOOSE_BTN    =   "filechooserbutton1"
    __ENTRY_FOLDER      =   "entry_folder1"
    __EXPANDER          =   "expander_options1"
    __PROGESS_BAR       =   "progressbar1_export"

    ###########################################################################
    ### PROPERTIES
    ########################################################################### 
  
    def _get_progress_bar_fraction(self):
        """ progress_bar_fraction getter """ 
        return self._progressbar.get_fraction()
    def _set_progress_bar_fraction(self, fraction): 
        """ progress_bar_fraction setter """ 
        if (fraction >= 0.0) and (fraction <= 1.0) :
            self._progressbar.set_fraction(fraction)
        else :
            # no value in range : just pulse
            self._progressbar.pulse()
    progress_bar_fraction = property(
                _get_progress_bar_fraction, 
                _set_progress_bar_fraction, 
                None , 
            "Progress bar fraction. value in range [0.0 .. 1.0] ")
  
    def _get_progress_bar_text(self): 
        """ progress_bar_text getter """ 
        return self._progressbar.get_text()
    def _set_progress_bar_text(self, text): 
        """ progress_bar_text getter """ 
        self._progressbar.set_text("%s"%text)
    progress_bar_text = property(
        _get_progress_bar_text, 
        _set_progress_bar_text, 
        None ,
        " Text to display in progress bar")

    def __init__(self, gui_objects ):
        """ Init of module """
        # init logger
        self.logger = logging.getLogger('luciole')
        self.__gui_objects = gui_objects
        
        self.__exit = False

        self._dialog = self.__gui_objects[self.__EXPORT_DIALOG] 
        self._dialog.connect('delete-event', self._export_close)

        self._combobox = self.__gui_objects[self.__COMBO_BOX]
        self._combobox.set_active(0)
        # get filename entry
        self._filename = self.__gui_objects[self.__ENTRY_FILENAME]
        self._filename.set_text("export") 

        # set initial folder 
        self._filechooser = self.__gui_objects[self.__FILECHOOSE_BTN]
        self._filechooser.connect(  'current-folder-changed', 
                                    self._current_folder_changed
                                    )

        initial_folder = os.path.expandvars('$HOME')  
        self._filechooser.set_current_folder(initial_folder)
        self._entry_folder = self.__gui_objects[self.__ENTRY_FOLDER]
        self._entry_folder.set_text(initial_folder) 
        #expander for options
        self._expander = self.__gui_objects[self.__EXPANDER]
        self._expander.set_label('Options')

        self._export_dir = None     # export dir set up    
  
        self._progressbar =  self.__gui_objects[self.__PROGESS_BAR]
 
 
    ########################################################################### 
    ### CALLBACKS
    ########################################################################### 
    def _export_close(self,widget, event) :
        """  close action """
        self._dialog.hide()
        self.__exit = True
        return True

    def _current_folder_changed(self, widget) :
        """ New directory selected update folder entry."""
        self._entry_folder.set_text(widget.get_filename())
        self._export_dir = widget.get_filename()

        
    ########################################################################### 
    ### PUBLIC METHODS
    ########################################################################### 
  
    def run(self , project_data = None) :
        """ export window handling """
        # a project exists and loaded, the export Window can be opened
        #Update entry folder with _project_dir
        export_dir =  os.path.join( project_data['project_dir'], 
                                    project_data['export_dir'])
        # Test if export dir has changed 
        # i.e . Project has changed --> in that case update the export dir.
        if (self._export_dir != export_dir) :
            self._entry_folder.set_text(export_dir)
            self._filechooser.set_current_folder(export_dir)
            self._export_dir = export_dir

        # clear progress bar .
        self._progressbar.set_fraction(0.0)
        self._progressbar.set_text('')
        
        # create an export object : Need the project tmp dir and widget 
        # to interact with 
        _exporter = LEXP.lcl_export_video( 
                                os.path.join(   project_data['project_dir'],
                                                project_data['tmp_dir']), 
                                self
                                    )
        
        # create export_data dictionnary who goes to collect data 
        # needed for export tools
        export_data = {}

        #set nb of frame per Image
        export_data['fpi'] = project_data['fpi'] 
        export_data['export_dir'] = self._export_dir
        
        # loop on exit for aplly button , when apply is clicked
        # the video export is done but the window is not closed
        # other reponse from gui can be handled
        self.__exit = False
        while (self.__exit == False) : 
            response = self._dialog.run()  #wait for a response 
            
            if response == gtk.RESPONSE_CANCEL:
                if _exporter.export_on_progress == False : 
                    # no export close export window
                    self._dialog.hide()
                    self.__exit = True
                else :
                    # export stop export progression
                    _exporter.cancel_export()

            elif response == gtk.RESPONSE_APPLY:
                # export is requested : collect info for export

                #
                # get image list from chrono/montage
                #

                # export data is type IMAGE_LIST
                export_data['image_input'] = LEXP.IMAGE_LIST      
                
                # Verify that a video name is entered and correct
                l_pattern = re.compile(r'^\w+$') 
                l_video_name = self._filename.get_text()
                if l_pattern.match(l_video_name) :
                    # format is correct 
                    export_data['video_name'] = l_video_name 
                else :
                    msg = _("%s is not valid. Not valid video name. \n\
                            It should be a combination of alphanumeric \
                            and '_' characters ") % l_video_name 
                    G_DIALOG.Dialog.ErrorMessage(self._dialog, msg)
                    # by pass loop
                    continue
            
                # creation of image list  In  chrono_images the image names 
                # are listed, but the full path is need by export module
                # so each file is concated with abasolute pathname to rush_dir
                export_data['image_list'] = \
                    [ os.path.join( project_data['project_dir'], 
                                    project_data['rush_dir'], 
                                    image_name) \
                            for image_name in project_data['chrono_images'] ]
          
                # get the type of export requested form the combobox
                _combo_val = self._combobox.get_active()
                if _combo_val == 0 :
                    _export_type = LEXP.EXPORT_DV
                elif _combo_val == 1 :
                    #export do dvd  
                    _export_type = LEXP.EXPORT_DVD
                elif _combo_val == 2 :
                    #export do xvid
                    _export_type = LEXP.EXPORT_XVID
                else :
                    self.logger.info('unknown video export command')
                    _export_type = None
                export_data['export_type'] = _export_type
               

                # update export dir
                export_data['export_dir'] = self._export_dir
                # launch Export action
                
                # by default overide of export filename is not allowed
                _force_export = False     
                (_res_export, _videopath) = \
                        _exporter.export(export_data , False)
                if (_res_export == LEXP.ERR_FILE_EXIST) :
                    # Launch Dialog window to ask if export file can be overide
                    msg = _("File %s already exists. Replace file ?") % \
                            _videopath
                    _force_export = G_DIALOG.Dialog.question_message(
                                            self._dialog, 
                                            msg
                                            )
                    # if response is True replace file, else do nothing.
                    if _force_export == True :
                        (_res_export, _videopath) = \
                            _exporter.export(export_data , True)
            
            elif response == gtk.RESPONSE_CLOSE :
                #close export window
                self._dialog.hide()
                self.__exit = True
 
    
      


