#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
    Main interface for acquisition
"""
# standard library imports
import os.path
import time

# related third party imports
import pygtk
pygtk.require("2.0")

# local application/library specific imports
import luciole.media.gst.acq as LGA
import luciole.constants as LCONST


class Acquisition(object) :
    """ Handle the acquisition from digital devices """
 
    ALPHA_DEFAULT = LCONST.ALPHA_DEFAULT
 
    #########################################################################
    # class properties
    #########################################################################

    def __get_is_onion_skin_active(self) :
        """ getter for is_onion_skin_active """ 
        return self._is_onion_skin_active
    def __set_is_onion_skin_active(self, value) :
        """ setter for is_onion_skin_active """ 
        self._is_onion_skin_active = value
    is_onion_skin_active = property(
            __get_is_onion_skin_active,
            __set_is_onion_skin_active,
            None,
            "Onion Skin activity " )
    
    def __get_image2mix(self) :
        """ getter for image2mix """ 
        return self.__image2mix
    def __set_image2mix(self, image_path) : 
        """ setter for image2mix """ 
        if ( os.path.isfile(image_path)) : 
            self.__image2mix = image_path
            if ( self._is_onion_skin_active) and ( self._gst_obj != None ) : 
                self._gst_obj.image2mix =  self.__image2mix

    image2mix = property(
            __get_image2mix,
            __set_image2mix,
            None,
            " Image to mix on Mixer" )
  
    def __get_is_streaming_active(self) :
        """ getter for is_streaming_active """ 
        return self._is_streaming_active
    is_streaming_active = property(
            __get_is_streaming_active,
            None,
            None,
            "Streaming  activity " )

    def __get_hard_type(self) :
        """ getter for hard_type """ 
        return self._hard_type
    hard_type = property(
            __get_hard_type,
            None,
            None,
            "Hard type for acquisition " )
  
    def __init__(self,
            display_widget,
            is_onion_skin_active = False,
            hard_type=LCONST.FAKE,
            project_dir = None,
            cb_error = None,
            cb_capture_done = None ): 
        """ Init of class Acquisition :
            inputs :
          - display_widget : The gtk.DrawingArea object for display the 
                            acquired stream from video
          - is_onion_skin_active : Is onion skin need to be activated
          - hard_type : Type of video to acquire 
          - project_dir : path to the project dir
          - cb_error : callback to indicate gstreamer error
          - cb_capture_done :  callback to indicate is done and a file is
                                available
        """
        self._hard_type = hard_type
        self.__project_dir = project_dir

        self._ctrller_on_error = cb_error
        self._cb_capture_done = cb_capture_done


        if self._hard_type != LCONST.DIGICAM :
            # Acquisition is gstreamer based
            self._gst_obj = LGA.Acquisition(  display_widget,
                                            self.__project_dir,
                                            self._cb_on_error,
                                            self._cb_capture_done)

            self.image2save = os.path.join( project_dir,
                                            LCONST.ACQUIRED_IMAGE_NAME)
            #No image to mix at creation
            self.__image2mix = None     

            #set Gstreamer default properties
            self._gst_obj.input_type =  self._hard_type 

            # Is onion skin active ?
            self._is_onion_skin_active = is_onion_skin_active     
            self._gst_obj.mix = LGA.Acquisition.NOMIX


            self._gst_obj.capture_image_path = self.image2save

    
        else :
            # Acquistion is based on image import
            self._is_onion_skin_active = False

        self._is_streaming_active = False
      
    def start_acquisition(self) : 
        """ start the acquisition on digital device"""

        #test if onion skin is yet requested
        if (self._is_onion_skin_active) and (self.__image2mix) :
            #set gstreamer object with onion skin
            self._gst_obj.mix = LGA.Acquisition.MIX
            self._gst_obj.alpha_image = 0.4
            self._gst_obj.image2mix =  self.__image2mix
        self._gst_obj.reset_pipe()  
        # nbd@grape : test play return failure or not
        self._gst_obj.play()         # start playing
        self._is_streaming_active = True

    def stop_acquisition(self) :
        """ stop acquisition on video device """
        if self._gst_obj.is_playing():
            self._gst_obj.stop()         # stop playing
        self._is_streaming_active = False
     
    def capture_image(self) :
        """ capture/snapshot of an image from video device """ 
        self._gst_obj.capture()
  
    def active_onion_skin(self, img = None) : 
        """ activation of onion skin"""
        # When onion skin activation is set the image2mix have to be choosen
        # The logic of choice is :
        # first if an image was yet captured ( self.__image2mix Not empty)
        # if no image select, checl if an image is given as parameter  
        # if no image as parm image. get an image from stream by forcing 
        # a capture
        if ( self.__image2mix ) :
            self._gst_obj.image2mix =  self.__image2mix
        else :
            if (img) :
                self._gst_obj.image2mix = img
            else :
                # force image capture
                self._gst_obj.capture()
                # wait acquisition is done i.e. file self.image2save exists 
                # on dir. this wait is done for simplification . could be 
                # also done with  callback on_capture
                while not os.path.exists(self.image2save) :
                    time.sleep(0.01)
                self.__image2mix = self.image2save
                if ( self.__image2mix ) :
                    self._gst_obj.image2mix =  self.__image2mix
              
        self._gst_obj.mix = LGA.Acquisition.MIX
        self._gst_obj.alpha_image = self.ALPHA_DEFAULT
    
        if self._gst_obj.is_playing(): 
            self._gst_obj.stop()
        #self._gst_obj.reset_pipe()  # reset pipe to take into account the mix
        #self._gst_obj.play() 
    
        self._is_onion_skin_active = True
    
    def deactive_onion_skin(self) : 
        """ deactivation of onion skin """
        self._gst_obj.mix = LGA.Acquisition.NOMIX
        
        if self._gst_obj.is_playing(): 
            self._gst_obj.stop()

        
        self._is_onion_skin_active = False
  
    def set_alpha_onion_skin (self, value) : 
        """ Set alpha value on Image"""
        self._gst_obj.alpha_image = value
 
    def _cb_on_error(self, code =0, message="") :
        """ Error detected by gstremer during acquisition """
        self._is_streaming_active = False
        # controller callbacl for error
        if self._ctrller_on_error : 
            self._ctrller_on_error(message)
        
  
class AcquisitionDigicam(Acquisition) :
    """ 
    Heritage of Acquisition class to handle digicam device :
        Dummy class dor debug purpose only. 
    """
  
    def __init__(self, display):
        """ constructor :
            - display : The gtk.DrawingArea object for display the image 
                acquired from digicam. REMARK Is this needed ??
        """
        Acquisition.__init__(self, display, False, LCONST.DIGICAM, None)
  
class AcquisitionWebcam(Acquisition) :
    """ Heritage of luciole acquisition for webcam  usage """

    def __init__(   self, 
                    display_widget,
                    is_onion_skin_active = False,
                    data=None,
                    project_dir = None,
                    cb_error = None,
                    cb_capture_done = None): 
        """ module init as parent + new param (data) """
        Acquisition.__init__(   self, 
                                        display_widget,
                                        hard_type = LCONST.WEBCAM,
                                        project_dir = project_dir,
                                        cb_error = cb_error ,
                                        cb_capture_done = cb_capture_done )

        # add webcam data info to gst obj
        self._gst_obj.webcam_data = data
        
        

	

	
