#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
    acq.py:
    Manage gstreamer webcam and DV cam acquisition and display
"""
# standard library imports
import os.path

# related third party imports
import gobject

# local application/library specific imports
import gst_base as LG
import luciole.base.tools as LT
import luciole.constants as LCONST


class PhotoSaveBin(LG.Bin) :
    """ 
    Bin Pad to save in jpeg format image from stream. 
    Stream alyays encoded but the result will be saved 
    to file only when a capture is done 
    """
    
    # Do jpeg encoding with a framerate of 5 images per 
    # second used to reduce cpu
    JPEG_CAPS_FILTER =  " video/x-raw-yuv, framerate=5/1 "
    
    def __init__(self) :
        """
        Description of pipeline :
        sink pad | jpegenc -->  fakesink | 
        """
        LG.Bin.__init__(self)
        _bin_elems = []
        #  rate transformation
        _video_rate_bin = LG.element_factory_make('videorate')
        _bin_elems.append(_video_rate_bin)

        _jpeg_filter_bin = LG.element_factory_make("capsfilter", "JpegFilter")
        # create caps string according, width, height and framerate
        caps = LG.Caps(self.JPEG_CAPS_FILTER)

        self.logger.debug(" Webcam cap : %s"%caps.to_string())
        _jpeg_filter_bin.set_property("caps", caps)
        _bin_elems.append(_jpeg_filter_bin)
        
        # jpeg encoding
        _jpeg_enc_bin = LG.element_factory_make("jpegenc", "JpegEncoder")
        _bin_elems.append(_jpeg_enc_bin)   
        
        _photosink = LG.element_factory_make("fakesink", "PhotoSink")
        _bin_elems.append(_photosink)   
        
        for elem in _bin_elems : 
            self.add(elem)
        
        LG.element_link_many(   _video_rate_bin, 
                                _jpeg_filter_bin, 
                                _jpeg_enc_bin,
                                _photosink)
        
        self.add_pad(LG.GhostPad('sink', _video_rate_bin.get_pad('sink')))


class InputImageBin(LG.Bin, object):
    """ Load image to mix with stream"""
    def get_image2mix(self):
        """ image2mix getter """
        return self.__image2mix
    def set_image2mix(self, value): 
        """ image2mix setter """
        if ( os.path.isfile(value) ):
            self.__image2mix = value
    image2mix = property(get_image2mix,
                        set_image2mix,
                        None,
                        " Path to Image To Mix ")
  
    def get_framerate(self):
        """ framerate getter """
        return self.__framerate
    def set_framerate(self, value): 
        """ framerate setter """
        if ( os.path.isfile(value) ):
            self.__framerate = value
    framerate = property(   get_framerate,
                            set_framerate, None,
                            " Framerate for displaying input image ")

    def get_alpha_value(self): 
        """ alpha_value getter """
        return self.__alpha_value
    def set_alpha_value(self, value): 
        """ alpha_value setter """
        self.__alpha_value = value
        self._alpha.set_property("alpha", self.__alpha_value)    
    alpha_value = property( get_alpha_value,
                                set_alpha_value, None,
                                " Framerate for displaying input image ")

    def __init__(self, image, alpha_value, framerate=1) :
        """
        Description of pipeline :
        | multifilesrc --> capsfilter --> jpegdec --> videoscale --> 
            capsfilter --> alpha --> | src pad
        """
        LG.Bin.__init__(self)
        self.__image2mix = image
        self.__framerate =  framerate
        self.__alpha_value = alpha_value
        _bin_elems = []
    
        # load input image
        _input_image_bin = LG.element_factory_make('multifilesrc')
        _input_image_bin.set_property('location', self.__image2mix)
        _bin_elems.append(_input_image_bin)
       

        # filter to set frame rate
        _image_filter_bin = LG.element_factory_make("capsfilter",
                                                    "ImageFilter")
        self._filter_caps_string = \
            "image/jpeg, framerate=(fraction)%s/1 ,width=%s,height=%s "
        
        caps_string = self._filter_caps_string % (self.__framerate,
                                                LCONST.VIDEO_PAL_RES[0],
                                                LCONST.VIDEO_PAL_RES[1])
        caps = LG.Caps(caps_string)
        _image_filter_bin.set_property("caps", caps)
        _bin_elems.append(_image_filter_bin)
     
        # dec jpeg as stream
        _jpeg_dec_bin = LG.element_factory_make('jpegdec')
        _bin_elems.append(_jpeg_dec_bin)
    
        # scale image
        _video_scale_bin = LG.element_factory_make('videoscale') 
        _bin_elems.append(_video_scale_bin)
        
        # needed again for setting aspect ratio
        # set filter on scale
        _video_filter_bin = LG.element_factory_make( 'capsfilter', 
                                                    "VideoFilter")    
        caps2 = LG.Caps("video/x-raw-yuv, width=%s,height=%s"%\
                (LCONST.VIDEO_PAL_RES[0], LCONST.VIDEO_PAL_RES[1]))
        _video_filter_bin.set_property("caps", caps2)
        _bin_elems.append(_video_filter_bin)
        
        # set alpha of image 
        self._alpha = LG.element_factory_make("alpha")
        self._alpha.set_property("alpha", self.__alpha_value )    
        _bin_elems.append(self._alpha)
        
        for elem in _bin_elems : 
            self.add(elem)
        
        LG.element_link_many(   _input_image_bin, 
                                _image_filter_bin, 
                                _jpeg_dec_bin,
                                _video_scale_bin,
                                _video_filter_bin,
                                self._alpha)
        
        self.add_pad(LG.GhostPad('src', self._alpha.get_pad('src'))) 



class MixStreamAndImage(LG.Bin, object) :
    """ Mix the input stream with an image """
    
    def __get_image2mix(self): 
        """ image2mix getter """
        return self.__image2mix
    def __set_image2mix(self, value): 
        """ image2mix setter """
        if ( os.path.isfile(value) ):
            self.__image2mix = value
            self._image_bin.image2mix = self.__image2mix
    image2mix = property(__get_image2mix, __set_image2mix, None,
                            " Path to Image To Mix ")

    def get_alpha_value(self): 
        """ alpha_value getter """
        return self.__alpha_value
    def set_alpha_value(self, value):
        """ alpha_value setter """
        self.__alpha_value = value
        self._image_bin.alpha_value = self.__alpha_value
    alpha_value = property(get_alpha_value, set_alpha_value, None,
                                " alpha value to change. ")

    _CAPSFILTER_STR = \
        "video/x-raw-yuv, framerate=(fraction)10/1, width=%s, height=%s" % \
        (LCONST.VIDEO_PAL_RES[0],LCONST.VIDEO_PAL_RES[1] 
                )
    
    def __init__(self, alpha_value_stream, image, alpha_value) :
        """
        Description of pipeline :
        sink pad | --> queue --> videorate --> videoscale --> capsfilter --> 
            videobox --> videoMixer | src pad
        """
        LG.Bin.__init__(self)
        
        self.__image2mix = image
        self.__alpha_value = alpha_value
        _bin_elems = []

        # queue  Bin 
        _queue_bin = LG.element_factory_make("queue", "queue_MixStreamAndImage")
        _bin_elems.append(_queue_bin)

        #  rate transformation
        _video_rate_bin = LG.element_factory_make('videorate')
        _bin_elems.append(_video_rate_bin)

        # scale image
        _video_scale_bin = LG.element_factory_make('videoscale')
        _bin_elems.append(_video_scale_bin)
        
        # scale image with caps
        _video_filter_bin = LG.element_factory_make("capsfilter")
        caps = LG.Caps(self._CAPSFILTER_STR) 
        _video_filter_bin.set_property("caps", caps)
        _bin_elems.append(_video_filter_bin)

        # videoBox where input video stream is displayed
        _video_box_bin = LG.element_factory_make('videobox', 
                                                 'VideoBox_MixSreamAndImage')
        _video_box_bin.set_property('border-alpha', 0)
        _video_box_bin.set_property('alpha', alpha_value_stream)
          
        _bin_elems.append(_video_box_bin)
        
        # Video mixer : mix input in videobox with image
        _video_mixer_bin = LG.element_factory_make('videomixer', 
                                              "VideoMixer_SreamAndImage")
        _bin_elems.append(_video_mixer_bin)
      
        # input image to mix
        # create Bin for input Image
        self._image_bin = InputImageBin(self.__image2mix, self.__alpha_value)
        _bin_elems.append(self._image_bin)
        
        for elem in _bin_elems : 
            self.add(elem)
        
        #input stream
        LG.element_link_many(   _queue_bin,
                                _video_rate_bin,
                                _video_scale_bin,
                                _video_filter_bin,
                                _video_box_bin,
                                _video_mixer_bin)
        # mix with image
        self._image_bin.link(_video_mixer_bin)
    
        # this module has a sink pad and a src pad
        self.add_pad(LG.GhostPad('sink', _queue_bin.get_pad('sink'))) 
        self.add_pad(LG.GhostPad('src', _video_mixer_bin.get_pad('src')))  




class ScaleBin(LG.Bin) :
    """ 
    This Bin allow to fit a display in fixed size with preserving aspect ratio 
    """
    
    def __init__(self, width=640, height=480) :
        """
        Description of pipeline : 
        sink pad | --> videoscale -->  capsfilter  --> videobox -->  | src pad
        """

        LG.Bin.__init__(self)
        _bin_elems = []
       
        # compute the image size if neeeded    
        image_size_d = self.calc_video_size(    float(width),
                                                float(height),
                                                LCONST.VIDEO_PAL_RES[0],
                                                LCONST.VIDEO_PAL_RES[1])

        # scale image
        _video_scale_bin = LG.element_factory_make('videoscale')
        _bin_elems.append(_video_scale_bin)

        # scale image with caps
        _video_filter_bin = LG.element_factory_make("capsfilter")
        caps = LG.Caps("video/x-raw-yuv,width=%s,height=%s"% \
                (image_size_d['width'], image_size_d['height'])) 
        _video_filter_bin.set_property("caps", caps)
        _bin_elems.append(_video_filter_bin)

        # videoBox where input video stream is displayed
        _video_box_bin = LG.element_factory_make('videobox', "VideoBox_Scale")
        _video_box_bin.set_property('fill', 0 )        # fill with black
        # borders are negative values for videobox . 
        # postive value used for crop
        _video_box_bin.set_property('top', -1*image_size_d['top'] )
        _video_box_bin.set_property('bottom', -1*image_size_d['bottom'])
        _video_box_bin.set_property('left', -1*image_size_d['left'])
        _video_box_bin.set_property('right', -1*image_size_d['right'])
        _bin_elems.append(_video_box_bin)

        # add bins
        for elem in _bin_elems : 
            self.add(elem)
        
        # link bins
        LG.element_link_many(   _video_scale_bin,
                                _video_filter_bin,
                                _video_box_bin )
        # this module has a sink pad and a src pad
        self.add_pad(LG.GhostPad('sink', _video_scale_bin.get_pad('sink'))) 
        self.add_pad(LG.GhostPad('src', _video_box_bin.get_pad('src'))) 
 
    def calc_video_size(w_src, h_src, w_dst, h_dst):
        """ 
        Compute the scale for the image : 
        The new image size if needed to scale + border to fill in 
        target destination
        """ 
        w_scale = (w_src*1.0) / w_dst
        h_scale = (h_src*1.0) / h_dst
        if w_scale >  h_scale :
            scale_f = w_scale
        else  :
            scale_f = h_scale
        if scale_f > 1.0 :
            # Scale to fill in defined zone
            (w_final , h_final) =  (int(w_src/scale_f) , int(h_src/scale_f))
        else :
            # input source lower than target don't scale it
            (w_final , h_final) =  (int(w_src) , int(h_src))

        (border_left, border_right) = ScaleBin.calc_border_size(w_final, w_dst)
        (border_top, border_bottom) = ScaleBin.calc_border_size(h_final, h_dst)
        size_dict = dict()
        size_dict['width'] =  int(w_final)
        size_dict['height'] =  int(h_final)
        size_dict['bottom'] =  int(border_bottom)
        size_dict['top'] =  int(border_top)
        size_dict['left'] =  int(border_left)
        size_dict['right'] =  int(border_right)
        return size_dict 
    calc_video_size = staticmethod(calc_video_size) 

    def calc_border_size(src, tgt) :
        """ 
        This function computes the size of the borders.
        Images is centered so borders size are equals at one pixel
        """
        (border_x, border_y) = (0, 0)

        if src < tgt :
            border_size = int( (tgt - src)/2)
            if border_size*2 + src < tgt :
                # to esnsure that border + src = target size
                # means that border size or src are odd
                # so the first is filled with the missing pixel 
                border_x = int(tgt - src -border_size)
                border_y = border_size
            else :
                (border_x, border_y) = ( border_size, border_size)
    
        return (border_x, border_y)
    calc_border_size = staticmethod(calc_border_size) 


class WebcamInputBin(LG.Bin) :
    """ LG.Bin for Web cam input"""

    def __init__(self,  width=640,
                        height=480,
                        framerate=(2,25), 
                        source_input='v4l2src',
                        device='/dev/video0') :
        """build webcam cam input bin
        Description of pipeline : 
        | v4lsrc or v4l2src -->  capsfilter --> ffmpegcolorspace --> 
                                ScaleBin | src pad
        params :
            width :         The webcam width. Integer value
            height :        The webcam width. Integer value
            framerate :     The webcam framerate. A tuple for fraction 
                            representation (denom,num)
            source_input :  The type of driver input. A string
            device :        Path to webcam device. A string 

        """

        LG.Bin.__init__(self)
        _bin_elems = []
          
        _video_src_bin = LG.element_factory_make(source_input)
        _video_src_bin.set_property("device", device)
        _bin_elems.append(_video_src_bin)
        
        _video_filter_bin = \
            LG.element_factory_make("capsfilter", "filter_webcamInput")
        # create caps string according, width, height and framerate
        caps = LG.Caps(\
                "video/x-raw-yuv, width=%s,height=%s , framerate=%s/%s"\
                    %(width, height, framerate[0], framerate[1]))
        self.logger.debug(" Webcam cap : %s"%caps.to_string())
        _video_filter_bin.set_property("caps", caps)
        _bin_elems.append(_video_filter_bin)
        
        _ffmpegcolorspace_bin = LG.element_factory_make('ffmpegcolorspace')
        _bin_elems.append(_ffmpegcolorspace_bin)
        
        # scale Webcam Display to DV SCALE 
        _scale_bin = ScaleBin(width, height)
        _bin_elems.append(_scale_bin)
                
        for elem in _bin_elems :
            self.add(elem)
        
        LG.element_link_many(
                            _video_src_bin,
                            _video_filter_bin,
                            _ffmpegcolorspace_bin,
                            _scale_bin)

        #this bin has only a src ghostPad
        self.add_pad(LG.GhostPad('src', _scale_bin.get_pad('src'))) 
        
     
      
class DvInputBin(LG.Bin) :
    """ LG.Bin for DV cam input"""
    def __init__(self) :
        """build Dv cam input bin
        Description of pipeline : 
        | dv1394src -->  dvdemux --> dvdec --> ffmpegcolorspace --> | src_pad
         
        """
        LG.Bin.__init__(self)
        _bin_elems = []
        
        _video_src_bin = LG.element_factory_make('dv1394src')
        _bin_elems.append(_video_src_bin)
        
        #set demuxer ( separation image/sound)
        _demux_bin = LG.element_factory_make("dvdemux", "demuxer") 
        _bin_elems.append(_demux_bin)
        _demux_bin.connect("pad-added", self._demux_bin_callback)
    
        self._decoder_bin = LG.element_factory_make("dvdec", "DV-decoder")
        _bin_elems.append(self._decoder_bin)
    
        # ffmpegcolorspace  needed for pipe 
        _ffmpegcolorspace_bin = LG.element_factory_make('ffmpegcolorspace')
        _bin_elems.append(_ffmpegcolorspace_bin)
        
        for elem in _bin_elems :
            self.add(elem)
        
        LG.element_link_many(   _video_src_bin,
                                _demux_bin)
        LG.element_link_many(   self._decoder_bin,
                                _ffmpegcolorspace_bin)
        
        #this bin has only a src ghostPad
        self.add_pad(
            LG.GhostPad('src', _ffmpegcolorspace_bin.get_pad('src'))
            ) 

    
    def _demux_bin_callback(self, demuxer, pad):
        """ Call back function to create the video pad of dvdemux."""  
        if pad.get_property("template").name_template == "video":
            dec_pad = self._decoder_bin.get_pad("sink")
            pad.link(dec_pad)
    
          
class FakeInputBin(LG.Bin) :
    """ LG.Bin for fake input"""
    def __init__(self) :
        """build Fake input bin.
        Description of pipeline : 
        | videotestsrc -->  capsfilter --> | src pad
         
        """
        LG.Bin.__init__(self)

        _bin_elems = []
        
        _video_src_bin = LG.element_factory_make('videotestsrc')
        _bin_elems.append(_video_src_bin)
        
        _video_filter_bin = LG.element_factory_make("capsfilter", 
                                                    "filter_fakeInput")
        caps = LG.Caps(\
            "video/x-raw-yuv, framerate=(fraction)10/1, width=720,height=576")
        _video_filter_bin.set_property("caps", caps)
        _bin_elems.append(_video_filter_bin)
        
        for elem in _bin_elems :
            self.add(elem)
        
        LG.element_link_many(   _video_src_bin,
                                _video_filter_bin)
        
        #this bin has only a src ghostPad
        self.add_pad(LG.GhostPad('src', _video_filter_bin.get_pad('src'))) 
        

class DisplayBin(LG.Bin) :
    """ 
    Bin used for video display on screen 
      Description of pipeline :

      sink pad | --> ffmpegcolorspace --> videoscale --> autovideosink | 

    """

    def __init__(self) :
        """ Build display bin """
        LG.Bin.__init__(self)
        _bin_elems = []

        # ffmpegcolorspace for display
        _ffmpegcolorspace_bin = LG.element_factory_make('ffmpegcolorspace')
        _bin_elems.append(_ffmpegcolorspace_bin)
        
        # scale image
        _video_scale_bin = LG.element_factory_make( 'videoscale',
                                                'DisplayVideoScale') 
        _bin_elems.append(_video_scale_bin)
        
        # sink video for display
        _video_sink_bin = LG.element_factory_make('autovideosink')
        _bin_elems.append(_video_sink_bin)

        for elem in _bin_elems :
            self.add(elem)
        
        LG.element_link_many(   _ffmpegcolorspace_bin,
                                _video_scale_bin,
                                _video_sink_bin)
        
        # only a sink (input) pad
        self.add_pad(
            LG.GhostPad('sink', _ffmpegcolorspace_bin.get_pad('sink'))
            )



class Acquisition(LG.LclGst) :
    """ Main class for hardware video acquisition with gstreamer"""      
                
    #
    # Class constants declaration
    # public constants
    (NOMIX, MIX) = range(2)
    # private constants
    __MixType = (NOMIX, MIX)
    __MIX_IMAGE_NAME = "ToMix.jpeg"
    __PIPE_NAME =  'Acquistion Pipe'

  
 
    #
    # Properties declaration
    #
    def __get_input_type(self):
        """ input_type getter """ 
        return self.__input_type
    def __set_input_type(self, value): 
        """ input_type setter """ 
        if value in LCONST.HARD_TYPE :
            self.__input_type = value
        else :
            msg = " Invalid Type : ", value, \
                    ". Not in ",  LCONST.HARD_TYPE
            self.logger.info(msg)
    def __del_input_type(self):
        """ input_type  """ 
        del self.__input_type
    input_type = property(__get_input_type, __set_input_type, __del_input_type,
            " Hard input type ")
  
    def __get_mix(self): 
        """ mix getter """ 
        return self.__mix
    def __set_mix(self, value): 
        """ mix setter """ 
        if value in self.__MixType :
            self.__mix = value
    def __del_mix(self): 
        """ mix deleter """ 
        del self.__mix
    mix = property(__get_mix, __set_mix, __del_mix, " Mix type ")

    def __get_capture_image_path(self): 
        """ capture_image_path getter """ 
        return self.__capture_image_path
    def __set_capture_image_path(self, value): 
        """ capture_image_path setter """ 
        self.__capture_image_path = value
    def __del_capture_image_path(self): 
        """ capture_image_path deleter """ 
        del self.__capture_image_path
    capture_image_path = property(  __get_capture_image_path,
                                    __set_capture_image_path,
                                    __del_capture_image_path,
                                    " Path to save file ")

    def __get_alpha_stream(self): 
        """ alpha_stream getter """ 
        return self.__alpha_stream
    def __set_alpha_stream(self, value): 
        """ alpha_stream getter """ 
        if ( (value >= 0.0) and  (value <= 1.0) ):
            self.__alpha_stream = value
    def __del_alpha_stream(self): 
        """ alpha_stream deleter """ 
        del self.__alpha_stream
    alpha_stream = property(    __get_alpha_stream, 
                                __set_alpha_stream,
                                __del_alpha_stream, 
                                "Stream alpha value ")
  
    def __get_alpha_image(self): 
        """ alpha_image getter """ 
        return self.__alpha_image
    def __set_alpha_image(self, value): 
        """ alpha_image setter """ 
        if ( (value >= 0.0) and  (value <= 1.0)) :
            self.__alpha_image = value
            if self._mix_bin : 
                self._mix_bin.alpha_value = self.__alpha_image
    def __del_alpha_image(self): 
        """ alpha_image deleter """ 
        del self.__alpha_image
    alpha_image = property( __get_alpha_image,
                            __set_alpha_image,
                            __del_alpha_image, 
                            " Image alpha value ")
  
    def __get_image2mix(self):
        """ image2mix getter """ 
        return self.__image2mix
    def __set_image2mix(self, value): 
        """ image2mix getter """ 
        if ( os.path.isfile(value) ):
            LT.copyf(value, self.__image2mix)    
            if self._mix_bin : 
                self._mix_bin.image2mix = self.__image2mix
    def __del_image2mix(self): 
        """ image2mix deleter """ 
        del self.__image2mix
    image2mix = property(   __get_image2mix,
                            __set_image2mix,
                            __del_image2mix, 
                            " Path to Image To Mix ")

    def __get_webcam_data(self):
        """ webcam_data getter """ 
        return self.__webcam_data
    def __set_webcam_data(self, value):
        """ webcam_data setter """ 
        self.__webcam_data = value 
    def __del_webcam_data(self):
        """ webcam_data deleter """ 
        del self.__webcam_data
    webcam_data = property( __get_webcam_data,
                            __set_webcam_data, 
                            __del_webcam_data, 
                            " webcam parameters for gstreamer :\
                                source_input,device,height, width ")
  
    def __init__(self,  videowidget,
                        base_dir_path = "/dev/tmp",
                        on_error = None,
                        cb_capture_done = None) :
        """ Initialisation of class LucioleGstreamer 
            Input parameters :
            - videowidget : the widget into display acquisition
            - base_dir_path : a base path to store the image to mix
            - cb_on_error : callback to inidicate error
            - cb_capture_done : callback to indicate that capture is done.
        """
        
        
        LG.LclGst.__init__(self,    video_widget = videowidget,
                                    pipe_name = self.__PIPE_NAME,
                                    cb_on_error = on_error  )

        self.__input_type = LCONST.FAKE
        self.__alpha_stream = 1.0
        self.__alpha_image = 0.5
        self.__mix  = self.NOMIX
        self.__image2mix = os.path.join(base_dir_path, self.__MIX_IMAGE_NAME)
        self.__capture_image_path = ""

        self._cb_capture_done = cb_capture_done
        self._grabprobe = None

        self.playing = False
        self._mix_bin = None

        # init webcam data - set default values
        #  standard vga
        self.__webcam_data = {}
        self.__webcam_data['width'] = 640
        self.__webcam_data['height'] = 480
        self.__webcam_data['device'] = "/dev/video0"
        self.__webcam_data['source_input'] = "v4l2src"
        self.__webcam_data['framerate_list'] = [ (25, 2) ] 
        self.__webcam_data['framerate_selected'] = (25, 2) 

    def reset_pipe(self): 
        """ Gstreamer pipe configuration :
        Build the elements according acquisition input, 
        availablity of mixer, and set output to videop display and file.        
        """
        _element_list = []
        # reset the pipe on parrent class
        LG.LclGst.reset_pipe(self)

        if (self.__input_type == LCONST.WEBCAM) :
            _input_bin = WebcamInputBin(
                        width = self.__webcam_data['width'],
                        height = self.__webcam_data['height'],
                        framerate = self.__webcam_data['framerate_selected'],
                        source_input = self.__webcam_data['source_input'],
                        device = self.__webcam_data['device']
                        )
        elif (self.__input_type == LCONST.DVCAM) :
            _input_bin = DvInputBin()
        else :
            _input_bin = FakeInputBin()
        _element_list.append(_input_bin)
   
        #create tee ( One for file sink, the other for video sink)
        _tee_bin = LG.element_factory_make("tee", "Tee")
        _element_list.append(_tee_bin)
    
        # both "branch" of the tee are queued
        
        #    
        # File queue
        #    
        _queue_file_bin = \
            LG.element_factory_make("queue","queueFile")              
        _element_list.append(_queue_file_bin)
        _file_sink_bin = PhotoSaveBin()
        _element_list.append(_file_sink_bin)
        
        #
        # Display queue
        #
        _queue_display_bin = \
            LG.element_factory_make("queue","queueDisplay")        
        _element_list.append(_queue_display_bin)
        if (self.__mix == self.MIX) :
            self._mix_bin = MixStreamAndImage( self.__alpha_stream,
                                            self.__image2mix,
                                            self.__alpha_image)
            _element_list.append(self._mix_bin)

        _display_sink_bin =  DisplayBin()
        _element_list.append(_display_sink_bin)

        #
        # Add elements to pipeline
        # self.pipe is set in parent class
        for _elem in _element_list : 
            self.pipe.add(_elem)

        #
        # link pipeline elements
        #
        #link input
        LG.element_link_many(   _input_bin,
                                _tee_bin)
        # link  tee File branch 
        LG.element_link_many(   _tee_bin,
                                _queue_file_bin,
                                _file_sink_bin)
        #link tee display Branch
        LG.element_link_many(   _tee_bin,
                                _queue_display_bin)
        
        # link with mixer if activated
        if (self.__mix == self.MIX) :
            LG.element_link_many(   _queue_display_bin,
                                    self._mix_bin,
                                    _display_sink_bin)
        else :
            LG.element_link_many(   _queue_display_bin,
                                    _display_sink_bin)
    
        
        # connect bus
        LG.LclGst.connect_bus(self)

    
    def capture(self) :
        """ capture is requested : 
            - get jpegnec src pad (output pad of jpege enc)
            - and add a buffer probe with callback when frame is encoded
        """
        # only one src pad with jpegenc
        pad = self.pipe.get_by_name("JpegEncoder").src_pads().next()
        # add the probe
        self._grabprobe = pad.add_buffer_probe(self._cb_process_frame)
    
        
    def _cb_process_frame(self, pad, gst_buffer):
        """ Callbak to inidicate that buffer is available from probe """
        # remove the prove
        pad.remove_buffer_probe(self._grabprobe)
        
        # save to file 
        _file = open(self.__capture_image_path,'w')
        _file.write(gst_buffer)
        _file.close()
        
        # callback to indicate that job is done
        # the execution here is not in the same thread as gui ,
        # so call capture_done with idle add
        if self._cb_capture_done != None :
            gobject.idle_add(self._cb_capture_done) 

        return True						

