#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
play.py :
    play mounted movie
"""


# standard library imports
# N/A

# related third party imports
# N/A

# local application/library specific imports
import luciole.media.gst.gst_base as LG



class Player(LG.LclGst) :
    """ implementation of image per image video playing with gstreamer .
       Description of pipeline : 
    --> | multifilesrc --> capsfilter(framerate, image ratio) --> decodebin 
            --> ffmpegcolorspace -->xvimagesink |
    """

    __FILTER_CAPS_STRING = \
        "image/jpeg, framerate=(fraction)%s/1 ,width=360,height=288"
    
    def __get_nb_images(self):
        """ nb_images getter """ 
        return self.__nb_images
    def __set_nb_images(self, value):
        """ nb_images setter """ 
        # check value is int and source bin exists
        if isinstance(value, int) and self._image_src_bin :
            self.__nb_images = value   
            self._image_src_bin.set_property('num-buffers', self.__nb_images)
    nb_images = property(   __get_nb_images,
                            __set_nb_images,
                            None,
                            "Number of image to play")
 
    def __get_framerate(self):
        """ framerate getter """ 
        return self.__framerate
    def __set_framerate(self, value):
        """ framerate setter """ 
        # check value is string and filter bin exists
        if isinstance(value, str) and self._image_filter_bin :
            # update filter with framerate value
            caps = LG.Caps(self.__FILTER_CAPS_STRING % value)
            self._image_filter_bin.set_property("caps", caps)
            self.__framerate = value   
    framerate = property(   __get_framerate,
                            __set_framerate, 
                            None, 
                            "Framerate for displaying image ")

  
    def __init__(self,  video_widget,
                        location,
                        framerate="5",
                        nb_images= -1,
                        start_buf=0,
                        cb_on_eos=None) :

        """ init of gstreamer player 
          inputs :
        - video_widget : Drawing area widgert, in gstreamer style for playing video
        - location : where the image are stored 
        - framerate : the video framerate
        - nbImages  : Number of images to play
        - start_buf : index of the first image to play  
        - cb_on_eos : callback for EOS signal
        """
        LG.LclGst.__init__(self,    video_widget = video_widget,
                                    pipe_name = 'Play pipe',
                                    cb_on_eos = cb_on_eos  )

        self.__nb_images = nb_images
        self.__framerate = framerate
        
        self._location = location
        self._start_buf = start_buf

        _element_list = []

        # image load
        self._image_src_bin = LG.element_factory_make('multifilesrc')
        self._image_src_bin.set_property('location', self._location)
        self._image_src_bin.set_property('num-buffers', self.__nb_images)
        self._image_src_bin.set_property('index', self._start_buf)
        _element_list.append(self._image_src_bin)
    
        # filter
        self._image_filter_bin = LG.element_factory_make("capsfilter")
        _caps = LG.gst.Caps(self.__FILTER_CAPS_STRING % self.__framerate)
        self._image_filter_bin.set_property("caps", _caps)
        _element_list.append(self._image_filter_bin)
    
        # decodebin
        self._decoder_bin = LG.element_factory_make('decodebin')
        # Connect handler for 'new-decoded-pad' signal
        self._decoder_bin.connect('new-decoded-pad', self._on_new_decoded_pad)
        _element_list.append(self._decoder_bin)
    
        # ffmpegclorspace
        self._ffmpegcolorspace_bin = \
            LG.element_factory_make('ffmpegcolorspace')
        # Reference used in self.on_new_decoded_pad()
        self.vpad = self._ffmpegcolorspace_bin.get_pad('sink')
        _element_list.append(self._ffmpegcolorspace_bin)
    
        # Image sink
        _video_sink_bin = LG.element_factory_make('xvimagesink')
        _element_list.append(_video_sink_bin)
        #
        # Add elements to pipeline
        #
        for _elem in _element_list : 
            self.pipe.add(_elem)
    
        # link all elements
        LG.element_link_many(self._image_src_bin, self._image_filter_bin)
        self._image_filter_bin.link(self._decoder_bin)
        LG.element_link_many(self._ffmpegcolorspace_bin, _video_sink_bin)
       
        LG.LclGst.connect_bus(self)
     
    def _on_new_decoded_pad(self, element, pad, last):
        """ call back to connect decodebin to videopad """
        _caps = pad.get_caps()
        name = _caps[0].get_name()
        if  name == 'video/x-raw-yuv':
            if not self.vpad.is_linked(): # Only link once
                pad.link(self.vpad)
        
  
    def stop(self):
        """ Stop the video """
        # call upper class stop
        LG.LclGst.stop(self)
        self._image_src_bin.set_property('index', 0)
      
    
 
