#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#

# standard library imports
import os.path

# related third party imports
# N/A

# local application/library specific imports
import luciole.base.exceptions as LEXCEP
import luciole.base.tools as LT
import luciole.constants as LCONST
import luciole.media.gst.play as LGST_P

class Player(object):
    """ Manage play of image per image video in the preview window """
  
    _local_tmp_dir = "play"
    _jpeg_format = "output-%05d.jpeg"

    def __init__(self, videowidget, cb_eos, tmp_dir) :
        """ 
        init of class Player :
        inputs :
            - videoWidget : Drawing area widget where video is played
            - cb_eos : callback for video eos (eos)
            - tmp_dir : path to a temp dir
        Exeption : 
            - raise if temp dirs cannot be created 
        """
        self._cb_eos = cb_eos
        self._tmp_dir = tmp_dir
      
        # create temp dir, where image go to be stored for play  
        _image_dir =  os.path.join(self._tmp_dir, self._local_tmp_dir)
        #clean or create temp dir 
        try : 
            self._clean_tmp_dir(_image_dir)
        except LEXCEP.LucioException, err :
            raise LEXCEP.LucioException , err
        else : 
            self.image_dir = _image_dir      
      
            # init gstreamer player
            _image_path = os.path.join(self.image_dir, self._jpeg_format)
            self.player = LGST_P.Player(videowidget,
                                        _image_path,
                                        cb_on_eos = self.on_eos)

    
    def on_eos(self) :
        """ action to do when gestreamer EOS( end of stream) occurs """ 
        
        #self.player.on_eos = lambda *x: on_eos() # Why this line ?   
        self.player.stop()
        self._cb_eos()


    def start_play(self, player_data) :
        """ starts playing 
            player_data is a dictionary with :
            image_list : the image list to play. each element of the list is a full image path
            fpi : the video framerate 
        """
        self._copy_file_to_play(player_data ) # prepare image to play
        
        # get framerate to be used 
        _in_fpi = player_data['fpi']
        _framerate = None
        for (_key, _value) in LCONST.VIDEO_FPS_TABLE.iteritems() :
            #  find the position 
            (_frm, _fpi) = _value 
            if _fpi == _in_fpi : 
                _framerate = _frm
        # update gstreamer playe number of images 
        self.player.framerate = _framerate  
        self.player.play()

    def stop_play(self) :
        """ Stop Video player """
        # stop gstreamer player 
        self.player.stop()


    def _copy_file_to_play(self, player_data) :
        """ 
        This method prepare the image to play. Preparation is :
            - copy in a ordered way images in tmp dir all the images to play
            - set the number of image to play
            - set the framerate   
        """
        for _idx, _image in enumerate(player_data['image_list']) :
            _path_src = _image
            _file_dest = self._jpeg_format % _idx
            _path_dest = os.path.join(self.image_dir, _file_dest)
            # delete image if image exist
            if os.path.exists(_path_dest) :
                try : 
                    LT.delf(_path_dest)
                except LEXCEP.LucioException, _err :
                    raise LEXCEP.LucioException, _err

            # copy image
            try : 
                LT.copyf(_path_src, _path_dest)
            except LEXCEP.LucioException, _err :
                raise LEXCEP.LucioException, _err

        
        # numbers of image to play
        self.player.nbImages = len(player_data['image_list']) 

     
    def _clean_tmp_dir(tmp_dir) :
        """ Clean or create  tmp dir if needed"""

        if not os.path.exists(tmp_dir) :
            #create it
            try :
                LT.mkdirs(tmp_dir)
            except LEXCEP.LucioException, _err :
                raise LEXCEP.LucioException, _err
        else :
            # directory exist, clean it
            _list = LT.files_in_dir(tmp_dir)
            for _file in _list :
                _file_path = os.path.join(tmp_dir, _file)
                try :
                    LT.delf(_file_path)
                except LEXCEP.LucioException, err :
                    raise LEXCEP.LucioException, err.message
        
    _clean_tmp_dir = staticmethod(_clean_tmp_dir)


