#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
    webcam_detection.py :
    Webcam detection module
"""

# standard library imports
import logging 

# related third party imports
import dbus
import dbus.glib
import gobject
import gst

# local application/library specific imports
import luciole.base.exceptions as LEXCEP




# Availables video sources for gstreamer
GSTREAMER_VIDEO_SOURCES = [
    "v4l2src",
    "v4lsrc"
  ]

 # constants used for framerate computation  
_NB_FRAMERATE = 4
_MAX_FRAMERATE = gst.Fraction(25, 1)

class WebcamDetection(object) :
    """ class in charge of detection of available webcams """

    def __get_webcam_devices(self) : 
        """ getter for webcam_devices"""
        return self._webcam_devices
    webcam_devices = property(  __get_webcam_devices,
                                None,
                                None,
                                "webcam devices description")
    
   
    def __init__(self) :
        """ class init"""
        # init logger
        self.logger = logging.getLogger('luciole')
        
        self._webcam_devices = list()
    
    ############################################################
    ### PUBLIC METHODS
    ############################################################
    def detect_webcam(self) :
        """ performs webcam detection . 
            returns number of webcam detected 
        """
        _status = self._hal_detection()
        if _status != 0 :
            # at least webcam was found , perform gstremer detection 
            _status = self._gst_detection()
    
        return _status

    def get_gst_best_input(self, webcam_device_index=0) :
        """ for selected webcam device returns the gst.Bin with  
        the best video quality .
        Criteria are :
            1) first select mimetype in the following order :    
                1) video/x-raw-yuv
                2) video/x-raw-rgb
                3) image/jpeg
            2) get the greatest resolution according video mimetype
                'width'     
                'height'
                'source_input'      -> v4lsrc or v4l2src
                'device '           -> device path ex. : /dev/video0
        The method returns a dict with the following keys :
            
        """
        _video_formats = \
            self._webcam_devices[webcam_device_index]\
                                ['webcam_data']['video_formats']         
        _mimetypes = ('video/x-raw-yuv', 'video/x-raw-rgb', 'image/jpeg')
        (_width, _height)=(None, None)
        for _mimetype in _mimetypes :

            if _video_formats.has_key(_mimetype) :
                # the resolution format are sorted by resolution  
                # the higher resolution is first in list
                _width = _video_formats[_mimetype][0]['width']
                _height = _video_formats[_mimetype][0]['height']
                _framerate =  _video_formats[_mimetype][0]['framerate']
                
                # leave loop when mimetype is found
                break
        _webcam_bin_data = dict()
        if _width != None :
            # a format was detected : prepare pipeline info
            _webcam_bin_data['width'] = _width 
            _webcam_bin_data['height'] = _height
            _webcam_bin_data['framerate_list'] = _framerate
            _webcam_bin_data['framerate_selected'] = \
                    _framerate[len(_framerate)/2]
            _webcam_bin_data['source_input'] = \
                    self._webcam_devices[webcam_device_index]['v4l_driver']
            _webcam_bin_data['device'] = \
                    self._webcam_devices[webcam_device_index]['device'] 
            _webcam_bin_data['name'] = \
                    self._webcam_devices[webcam_device_index]['name']

        return  _webcam_bin_data

            
            
               

    ############################################################
    ### PRIVATE METHODS
    ############################################################

    def _hal_detection(self) :
        """ performs hal detection """
    
        # get a connection to the system bus
        _bus = dbus.SystemBus ()

        # get a HAL object and an interface to HAL to make function calls
        _hal_obj = _bus.get_object(   'org.freedesktop.Hal', 
                                    '/org/freedesktop/Hal/Manager')
        _hal = dbus.Interface (_hal_obj, 'org.freedesktop.Hal.Manager')

        # find all devices that have the capability 'video4linux'
        _udis = _hal.FindDeviceByCapability ('video4linux')
        
        # loop on found device 
        for _udi in _udis :
            _webcam_device = dict()
            
            # get a device object
            _dev_obj = _bus.get_object ('org.freedesktop.Hal', _udi)
            
            # get an interface to the device
            _dev = dbus.Interface (_dev_obj, 'org.freedesktop.Hal.Device')
            
            
            _webcam_device["name"] = _dev.GetProperty ('info.product')
            if _webcam_device["name"] == None : 
                _webcam_device["name"] = "Unknown"
            _webcam_device["device"] = _dev.GetProperty ('video4linux.device')
        
            # detect if need to use v4lsrc or v4l2sec ; 
            # property :video4linux.version
            # for robustness if version = 1 , than use v4lsrc else use v4l2src 
            _version = _dev.GetProperty ('video4linux.version')
            # default is v4l2src
            _webcam_device['v4l_driver'] = 'v4l2src'
            if _version != None and int(_version) == 1 :
                _webcam_device['v4l_driver'] = 'v4lsrc'
            

            if (not  _webcam_device["device"] == None) and \
                    ( self.__is_device_readable(_webcam_device["device"]) ) :
                self._webcam_devices.append(_webcam_device)
       
        # the function returns the number of device found.
        _nb_device_found = 0
        if not self._webcam_devices == None :
            _nb_device_found = len(self._webcam_devices)

        return _nb_device_found

    def _gst_detection(self) :
        """ performs webcam type detection with gstreamer :
            _ detects what source yo use ( v4lsrc or v4l2src)
            _ resolution 
            _ video mime types
        """ 
        _nb_device_detected = 0
        for _webcam_device in self._webcam_devices :
            # for each detected device get the 
            # data (source type, mimetype, resolution) 
            (_status, _webcam_device_data) = \
                    self.__get_gst_webcam_data( _webcam_device["device"],
                                                _webcam_device["v4l_driver"])
            if _status[0] == gst.STATE_CHANGE_SUCCESS\
                    and _webcam_device_data != None :  
                # detection is success
                _webcam_device["webcam_data"] = _webcam_device_data
                _nb_device_detected = _nb_device_detected +1
            else :
                # gstreamer test unsuccesfull : test with other driver
                if _webcam_device["v4l_driver"] == 'v4lsrc' :
                    _webcam_device["v4l_driver"] = 'v4l2src'
                else :
                    _webcam_device["v4l_driver"] = 'v4lsrc'
                (_status, _webcam_device_data) = \
                    self.__get_gst_webcam_data( _webcam_device["device"],
                                                _webcam_device["v4l_driver"])
                if _status[0] == gst.STATE_CHANGE_SUCCESS and\
                        _webcam_device_data != None :  
                    # detection is success
                    _webcam_device["webcam_data"] = _webcam_device_data
                    _nb_device_detected = _nb_device_detected +1
                else :
                    #Nothing detected . Reset webcam device
                    _webcam_device = dict()
        return _nb_device_detected
            

    ############################################################
    ### PRIVATE STATIC METHODS
    ############################################################
    def __is_device_readable(device) :
        """ check if device is readable """
        _is_readable = False
        try :
            _fd = open(device,'r')
        except IOError, _err:
            _is_readable = False
        else :
            _is_readable = True
            _fd.close()
        return _is_readable
    __is_device_readable = staticmethod(__is_device_readable)

    def __get_gst_webcam_data(device_name, driver) :
        """ get data from the webcam test compatible sources
            input device_name is the file path ( ex. /dev/video0)
        """
        # creation of a fake gstreamer to test webcam with v4lsrc ot v4l2src
        _webcam_device = dict()
        
        _pipeline_desc = \
            "%s name=source device =%s ! fakesink" % (driver, device_name)
        _pipeline = gst.parse_launch(_pipeline_desc)
            
        #Start pipeline and check for state 
        _pipeline.set_state(gst.STATE_PLAYING)
        _ret = _pipeline.get_state(timeout=10000*gst.MSECOND)
            
        if _ret[0] == gst.STATE_CHANGE_SUCCESS :
            # webcam is detected succefully
            _pipeline.set_state(gst.STATE_PAUSED)
                
            # get source element name and webcam name 
            _src = _pipeline.get_by_name('source')
                
                
            # start detection of source video formats
            # video formats are accesible in the capabilities
            # of the source "src" pad 
            _pad = _src.get_pad("src")
            _caps = _pad.get_caps()
            WebcamDetection.\
                get_supported_video_formats(_webcam_device, _caps)
            # stop playing with webcam
            _pipeline.set_state(gst.STATE_NULL)
        return (_ret, _webcam_device)
    __get_gst_webcam_data = staticmethod(__get_gst_webcam_data)

    
    
    def get_int_range(structure, resolution_list) : 
        """ retrieve data formatted as int range """
        _resolution = dict()
        _is_int_range = False
        # check if result is in GST_TYPE_INT_RANGE format
        if (isinstance(structure["width"], gst.IntRange)):
            _is_int_range = True
            # when type is range --> some resolution points are 
            # created in the range value are *2 from min to max
            _width_cur =  structure["width"].low
            _height_cur =  structure["height"].low
            while ( 
            ( _width_cur   <=  structure["width"].high ) 
                and
            ( _height_cur  <=  structure["height"].high ) 
            ):
                            
                ( _resolution["width"] , _resolution["height"] ) = \
                    (_width_cur,_height_cur)  
                #store framerate 
                _resolution["framerate"] = None
                if structure.has_field("framerate") :
                    _resolution["framerate"] = structure["framerate"]
                    # Append resolution dict to list
                if not _resolution  in resolution_list :
                    resolution_list.append(_resolution)
                (_width_cur, _height_cur) = (_width_cur*2, _height_cur*2)
        return _is_int_range
    get_int_range = staticmethod(get_int_range)

    def get_type_int(structure, resolution_list) :
        """ retreive gobjet.TYPE_INT data """
        _resolution = dict()
        _is_type_int = False
        if  structure.has_field_typed("width", gobject.TYPE_INT) :
            _is_type_int = True
            # check resolution
            (_resolution["width"] , _resolution["height"] ) = \
                        (structure["width"], structure["height"])
            #check framerate
            _resolution["framerate"] = None
            if structure.has_field("framerate") :
                _resolution["framerate"] = \
                    WebcamDetection.gst_get_framerate(structure["framerate"])
            if not _resolution  in resolution_list :
                resolution_list.append(_resolution)
        return _is_type_int
    get_type_int = staticmethod(get_type_int)

    
    def get_supported_video_formats(webcam_device, caps) :
        """ get video mimetypes and resolution 
            Format example  
            webcam_device ["mimetypes"]      ==> ["video/x-raw-yuv"]     ==> ( 
                                                                            (["width"] = 320 , ["height"] = 240) 
                                                                            (["width"] = 640 , ["height"] = 480) 
                                                                            )
                                             ==> ["video/x-raw-rgb"]     ==> ( 
                                                                            (["width"] = 640 , ["height"] = 480) 
                                                                            )
        
        The input structures  are identified by the mime type and have the width and height as field
        some other fields as bpp , depth or framerate are also available but not used here.
        """
        _num_structures = caps.get_size()
        webcam_device["video_formats"] = dict()
        for _idx in range(_num_structures) :
            # loop on each strutcures
            _structure = caps[_idx]
            if (    _structure.has_name("video/x-raw-yuv") 
                or  _structure.has_name("video/x-raw-rgb")  
                or  _structure.has_name("image/jpeg")
                ) :
                # get resolution list or create it
                if not webcam_device["video_formats"].\
                        has_key(_structure.get_name()):
                    webcam_device["video_formats"][_structure.get_name()] =\
                            list()
                _resolution_list = \
                        webcam_device["video_formats"][_structure.get_name()]
                
                # take in acount only structure wih fields width and height
                if (_structure.has_field("width") and\
                        _structure.has_field("height")) :
                    _resolution = dict()
                    # check if result is in GST_TYPE_INT_RANGE format
                    _test = WebcamDetection.get_int_range(
                                _structure, 
                                _resolution_list
                                )
                    # check if result is in TYPE_INT format
                    if not _test :
                        _test = \
                               WebcamDetection.get_type_int(
                                _structure, 
                                _resolution_list
                                )

                    if not _test : 
                        # raise error
                        _err =  "unkown video type %s" % \
                            _structure.get_field_type("width")
                        raise LEXCEP.LucioException, _err  
        
        for _mimetype, _res_list in webcam_device["video_formats"].iteritems():
            # loop on video on formats to sort it
            # data are sorted by width , in reverse order; i.e max res is the
            # first in the list
            # for sorting see howto sort :
            # http://wiki.python.org/moin/SortingListsOfDictionaries
            from operator import itemgetter
            _result = sorted(_res_list, key=itemgetter('width'))
            _result.reverse()
            webcam_device["video_formats"][_mimetype] = _result
        # debug purpose
        #for _mimetype,res_list in webcam_device["video_formats"].iteritems():
        #    print "----- %s : %s"%(_mimetype,res_list)
    get_supported_video_formats = \
            staticmethod(get_supported_video_formats)

    def gst_get_framerate(framerate_obj):
        """ create a list of gst.Fraction framerates """ 
        _framerate = list()
        _framerate_trans = None
        if type(framerate_obj) == list :
            _framerate = framerate_obj
        elif isinstance(framerate_obj, gst.FractionRange) :
            # convert also to list of framerate
                        
            # limit max framerate 
            if framerate_obj.high < _MAX_FRAMERATE :
                framerate_obj.high = _MAX_FRAMERATE
            
            _new_framerate = framerate_obj.high
            for _idx in range(_NB_FRAMERATE) :
                _framerate.append(_new_framerate)
                _new_framerate = _new_framerate/2
            # at end of loop append the lowest framerate if not 0
            if framerate_obj.low != gst.Fraction(0, 1) :
                _framerate.append(framerate_obj.low)
        else :
            # raise error
            _err = " unable to detect framerate : unknown type : %s for "% \
                    (type(framerate_obj),framerate_obj)
            raise LEXCEP.LucioException, _err  
        # transform framerate in not gst format
        if _framerate != None :
            _framerate_trans = [ (_fraction.num, _fraction.denom) \
                    for _fraction in _framerate]
        return _framerate_trans
    gst_get_framerate = staticmethod(gst_get_framerate)

if __name__ == '__main__' :
    # TEST PURPOSE  : for webcam detection
    CAM_OBJ = WebcamDetection()
    VAL = CAM_OBJ.detect_webcam()
    print " found %s webCam device " % VAL

    #if val >0 :
    #    for device in CAM_OBJ.webcam_devices : 
    #        for k,j in device.iteritems() : print "%s : %s"%(k,j)
    #        print "---------------------------------------------------------"
    for i in range(VAL) :
    #for enumerate(index,device) in CAM_OBJ.webcam_devices :
        best = CAM_OBJ.get_gst_best_input(i)
        print "Best webcam resolution found " 
        print best
        print "\n \n"

